@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -45,7 +45,7 @@ contain references to widgets as those belong strictly to the :class:`GuiContext
 class GuiContext( object ):
     """
 The GUI context in which an action is running.  This object can contain
-references to widgets and other usefull information.  This object cannot
+references to widgets and other useful information.  This object cannot
 contain reference to anything database or model related, as those belong
 strictly to the :class:`ModelContext`
 
@@ -287,6 +287,13 @@ values for these attributes can reimplement the getter methods.
     The shortcut that can be used to trigger the action, this should be of 
     type :class:`camelot.core.utils.ugettext_lazy`
     
+.. attribute:: drop_mime_types
+
+    A list of strings with the mime types that can be used to trigger this
+    action by dropping objects on the related widget.  Example ::
+    
+        drop_mime_types = ['text/plain']
+    
 An action has two important methods that can be reimplemented.  These are 
 :meth:`model_run` for manipulations of the model and :meth:`gui_run` for
 direct manipulations of the user interface without a need to access the model.
@@ -298,6 +305,7 @@ direct manipulations of the user interface without a need to access the model.
     tooltip = None
     shortcut = None 
     modes = []
+    drop_mime_types = []
     
     def get_name( self ):
         """
@@ -342,8 +350,13 @@ direct manipulations of the user interface without a need to access the model.
         """
         from camelot.view.controls.progress_dialog import ProgressDialog
         progress_dialog = None
-        # only create a progress dialog if there is none yet
+        # only create a progress dialog if there is none yet, or if the
+        # existing dialog was canceled
+        LOGGER.debug( 'action gui run started' )
+        if gui_context.progress_dialog and gui_context.progress_dialog.wasCanceled():
+            gui_context.progress_dialog = None
         if gui_context.progress_dialog == None:
+            LOGGER.debug( 'create new progress dialog' )
             progress_dialog = ProgressDialog( unicode( self.verbose_name ) )
             gui_context.progress_dialog = progress_dialog
             #progress_dialog.show()
@@ -352,6 +365,7 @@ direct manipulations of the user interface without a need to access the model.
         if progress_dialog != None:
             progress_dialog.close()
             gui_context.progress_dialog = None
+        LOGGER.debug( 'gui run finished' )
         
     def get_state( self, model_context ):
         """
@@ -367,3 +381,4 @@ direct manipulations of the user interface without a need to access the model.
         state.tooltip = self.tooltip
         state.modes = self.modes
         return state
+
