@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -30,6 +30,7 @@ from camelot.view.art import Icon
 
 from application_action import ( ApplicationActionGuiContext,
                                  ApplicationActionModelContext )
+import list_action
 
 class FormActionModelContext( ApplicationActionModelContext ):
     """On top of the attributes of the 
@@ -65,10 +66,6 @@ class FormActionModelContext( ApplicationActionModelContext ):
         self.collection_count = 0
         self.selection_count = 1
         
-    @property
-    def session( self ):
-        return self._model.admin.get_query().session
-        
     def get_object( self ):
         """
         :return: the object currently displayed in the form, None if no object
@@ -184,6 +181,42 @@ class CloseForm( Action ):
                     yield action_steps.DeleteObject( obj )
                     admin.expunge( obj )
     
+class ToPreviousForm( list_action.ToPreviousRow ):
+    """Move to the previous form"""
+
+    def gui_run( self, gui_context ):
+        gui_context.view.to_previous()
+        
+    def get_state( self, model_context ):
+        return Action.get_state( self, model_context )
+    
+class ToFirstForm( list_action.ToFirstRow ):
+    """Move to the form"""
+    
+    def gui_run( self, gui_context ):
+        gui_context.view.to_first()
+
+    def get_state( self, model_context ):
+        return Action.get_state( self, model_context )
+    
+class ToNextForm( list_action.ToNextRow ):
+    """Move to the next form"""
+
+    def gui_run( self, gui_context ):
+        gui_context.view.to_next()
+
+    def get_state( self, model_context ):
+        return Action.get_state( self, model_context )
+    
+class ToLastForm( list_action.ToLastRow ):
+    """Move to the last form"""
+
+    def gui_run( self, gui_context ):
+        gui_context.view.to_last()
+
+    def get_state( self, model_context ):
+        return Action.get_state( self, model_context )
+    
 def structure_to_form_actions( structure ):
     """Convert a list of python objects to a list of form actions.  If the python
     object is a tuple, a CallMethod is constructed with this tuple as arguments.  If
@@ -197,3 +230,4 @@ def structure_to_form_actions( structure ):
         return CallMethod( o[0], o[1] )
 
     return [object_to_action( o ) for o in structure]
+
