@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -85,10 +85,6 @@ class ListActionModelContext( ApplicationActionModelContext ):
         self.selected_rows = []
         self.field_attributes = dict()
         
-    @property
-    def session( self ):
-        return self.admin.get_query().session
-        
     def get_selection( self, yield_per = None ):
         """
         :param yield_per: an integer number giving a hint on how many objects
@@ -155,20 +151,29 @@ class ListActionGuiContext( ApplicationActionGuiContext ):
 
     def create_model_context( self ):
         context = super( ListActionGuiContext, self ).create_model_context()
-        context._model = self.item_view.model()
         context.field_attributes = copy.copy( self.field_attributes )
-        context.current_row = self.item_view.currentIndex().row()
-        context.collection_count = context._model.rowCount()
+        current_row = None
+        model = None
+        collection_count = 0
         selection_count = 0
         selected_rows = []
-        selection = self.item_view.selectionModel().selection()
-        for i in range( len( selection ) ):
-            selection_range = selection[i]
-            rows_range = ( selection_range.top(), selection_range.bottom() )
-            selected_rows.append( rows_range )
-            selection_count += ( rows_range[1] - rows_range[0] ) + 1
+        if self.item_view != None:
+            current_row = self.item_view.currentIndex().row()
+            model = self.item_view.model()
+            if model != None:
+                collection_count = model.rowCount()
+            if self.item_view.selectionModel() != None:
+                selection = self.item_view.selectionModel().selection()
+                for i in range( len( selection ) ):
+                    selection_range = selection[i]
+                    rows_range = ( selection_range.top(), selection_range.bottom() )
+                    selected_rows.append( rows_range )
+                    selection_count += ( rows_range[1] - rows_range[0] ) + 1
         context.selection_count = selection_count
+        context.collection_count = collection_count
         context.selected_rows = selected_rows
+        context.current_row = current_row
+        context._model = model
         return context
         
     def copy( self, base_class = None ):
@@ -221,7 +226,7 @@ class CallMethod( Action ):
         return state
             
 class ListContextAction( Action ):
-    """An base class for actions that should only be enabled in the
+    """An base class for actions that should only be enabled if the
     gui_context is a :class:`ListActionModelContext`
     """
     
@@ -580,8 +585,11 @@ class PrintPreview( ListContextAction ):
         columns = model_context.admin.get_columns()
         
         table = []
+        getters = [field_attributes['getter'] for _field, field_attributes in columns]
+        to_strings = [field_attributes['to_string'] for _field, field_attributes in columns]
+        column_range = range( len( columns ) )
         for obj in model_context.get_collection():
-            table.append( [getattr( obj, col[0] ) for col in columns] )
+            table.append( [to_strings[i]( getters[i]( obj ) ) for i in column_range] )
         context = {
           'title': model_context.admin.get_verbose_name_plural(),
           'table': table,
@@ -733,3 +741,4 @@ class RemoveSelection( EditAction ):
             session = object_session( objects_to_remove[0] )
         model_context._model.remove_objects( objects_to_remove, delete = False )
         yield action_steps.FlushSession( session )
+
