@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -22,14 +22,16 @@
 #
 #  ============================================================================
 
-import os
+import itertools
 import logging
+import os
+
 logger = logging.getLogger('camelot.admin.application_admin')
 
 from PyQt4.QtCore import Qt
 from PyQt4 import QtCore, QtGui
 
-from camelot.admin.action import list_action, application_action
+from camelot.admin.action import application_action, form_action, list_action
 from camelot.core.utils import ugettext_lazy as _
 from camelot.view import art
 from camelot.view import database_selection
@@ -133,6 +135,12 @@ shortcut confusion and reduce the number of status updates.
     help_actions = [ application_action.ShowHelp(), ]
     export_actions = [ list_action.PrintPreview(),
                        list_action.ExportSpreadsheet() ]
+    form_toolbar_actions = [ form_action.CloseForm(),
+                             form_action.ToFirstForm(),
+                             form_action.ToPreviousForm(),
+                             form_action.ToNextForm(),
+                             form_action.ToLastForm(),
+                             application_action.Refresh() ]
     
     def __init__(self):
         """Construct an ApplicationAdmin object and register it as the 
@@ -157,7 +165,7 @@ shortcut confusion and reduce the number of status updates.
         self.admins[entity] = admin_class
 
     @model_function
-    def get_sections(self):
+    def get_sections( self ):
         """A list of :class:`camelot.admin.section.Section` objects,
         these are the sections to be displayed in the left panel.
         
@@ -168,6 +176,17 @@ shortcut confusion and reduce the number of status updates.
         return [ Section( _('Relations'), self ),
                  Section( _('Configuration'), self ),
                  ]
+    
+    def get_settings( self ):
+        """A :class:`QtCore.QSettings` object in which Camelot related settings
+        can be stored.  This object is inteded for Camelot internal use.  If an
+        application specific settings object is needed, simply construct one.
+        
+        :return: a :class:`QtCore.QSettings` object
+        """
+        settings = QtCore.QSettings()
+        settings.beginGroup( 'Camelot' )
+        return settings
         
     def get_application_admin( self ):
         """Get the :class:`ApplicationAdmin` class of this application, this
@@ -289,6 +308,18 @@ shortcut confusion and reduce the number of status updates.
         """
         return []
     
+    def get_form_toolbar_actions( self, toolbar_area ):
+        """
+        :param toolbar_area: an instance of :class:`Qt.ToolBarArea` indicating
+            where the toolbar actions will be positioned
+            
+        :return: a list of :class:`camelot.admin.action.base.Action` objects
+            that should be displayed on the toolbar of a form view.  return
+            None if no toolbar should be created.
+        """
+        if toolbar_area == Qt.TopToolBarArea:
+            return self.form_toolbar_actions
+        
     def get_main_menu( self ):
         """
         :return: a list of :class:`camelot.admin.menu.Menu` objects, or None if 
@@ -398,6 +429,15 @@ shortcut confusion and reduce the number of status updates.
 
         Have a look at the default implementation to use another stylesheet.
         """
+        #
+        # Try to load a custom QStyle, if that fails use a stylesheet from
+        # a file
+        #
+        try:
+            from PyTitan import QtnOfficeStyle
+            QtnOfficeStyle.setApplicationStyle( QtnOfficeStyle.Windows7Scenic )
+        except:
+            pass
         return art.read('stylesheet/office2007_blue.qss')
 
     def _load_translator_from_file( self, 
@@ -420,20 +460,52 @@ shortcut confusion and reduce the number of status updates.
         :param directory: the directory, relative to the module in which
             to look for translation files
         :param suffix: the suffix of the filename
+        :param search_delimiters: list of characters by which to split the file
+            name to search for variations of the file name
         :return: :keyword:None if unable to load the file, otherwise a
             :obj:`QtCore.QTranslator` object.
+            
+        This method tries to load all file names with or without suffix, and
+        with or without the part after the search delimiter.
         """
         from camelot.core.resources import resource_string
+
+        #
+        # split the directory names and file name
+        #
+        file_name_parts = [ file_name ]
+        head, tail = os.path.split( file_name_parts[0] )
+        while tail:
+            file_name_parts[0] = tail
+            file_name_parts = [ head ] + file_name_parts
+            head, tail = os.path.split( file_name_parts[0] )
+        #
+        # for each directory and file name, generate all possibilities
+        #
+        file_name_parts_possibilities = []
+        for file_name_part in file_name_parts:
+            part_possibilities = []
+            for search_delimiter in search_delimiters:
+                delimited_parts = file_name_part.split( search_delimiter )
+                for i in range( len( delimited_parts ) ):
+                    part_possibility = search_delimiter.join( delimited_parts[:len(delimited_parts)-i] )
+                    part_possibilities.append( part_possibility )
+            file_name_parts_possibilities.append( part_possibilities )
+        #
+        # make the combination of all those possibilities
+        #
         file_names = []
-        for search_delimiter in search_delimiters:
-            file_name_parts = file_name.split( search_delimiter )
-            for i in range( len(file_name_parts) ):
-                partial_file_name = search_delimiter.join( file_name_parts[:len(file_name_parts)-i] )
-                file_names.append( partial_file_name + suffix )
-                file_names.append( partial_file_name )
+        for parts_possibility in itertools.product( *file_name_parts_possibilities ):
+            file_name = os.path.join( *parts_possibility )
+            file_names.append( file_name )
+            file_names.append( file_name + suffix )
+        #
+        # now try all file names
+        #
         translations = None
         for file_name in file_names:
             try:
+                logger.debug( u'try %s'%file_name )
                 translations = resource_string( module_name, os.path.join(directory,file_name) )
                 break
             except IOError:
@@ -461,14 +533,13 @@ shortcut confusion and reduce the number of status updates.
         translators = []
         qt_translator = QtCore.QTranslator()
         locale_name = QtCore.QLocale().name()
-        language_name = locale_name.split('_')[0]
         logger.info( u'using locale %s'%locale_name )
         if qt_translator.load( "qt_" + locale_name,
-                              QtCore.QLibraryInfo.location( QtCore.QLibraryInfo.TranslationsPath ) ):
+                               QtCore.QLibraryInfo.location( QtCore.QLibraryInfo.TranslationsPath ) ):
             translators.append( qt_translator )
         camelot_translator = self._load_translator_from_file( 'camelot', 
-                                                              'camelot',
-                                                              'art/translations/%s/LC_MESSAGES/'%language_name )
+                                                              os.path.join( '%s/LC_MESSAGES/'%locale_name, 'camelot' ),
+                                                              'art/translations/' )
         if camelot_translator:
             translators.append( camelot_translator )
         else:
@@ -502,7 +573,6 @@ shortcut confusion and reduce the number of status updates.
         """
         import sys
         import sqlalchemy
-        import elixir
         import chardet
         import jinja2
         import xlrd
@@ -512,7 +582,6 @@ shortcut confusion and reduce the number of status updates.
                   <em>Qt:</em> <b>%s</b><br>
                   <em>PyQt:</em> <b>%s</b><br>
                   <em>SQLAlchemy:</em> <b>%s</b><br>
-                  <em>Elixir:</em> <b>%s</b><br>
                   <em>Chardet:</em> <b>%s</b><br>
                   <em>Jinja:</em> <b>%s</b><br>
                   <em>xlrd:</em> <b>%s</b><br>
@@ -521,7 +590,6 @@ shortcut confusion and reduce the number of status updates.
                                               float('.'.join(str(QtCore.QT_VERSION_STR).split('.')[0:2])),
                                               QtCore.PYQT_VERSION_STR,
                                               sqlalchemy.__version__,
-                                              elixir.__version__,
                                               chardet.__version__,
                                               jinja2.__version__,
                                               xlrd.__VERSION__,
@@ -581,3 +649,4 @@ shortcut confusion and reduce the number of status updates.
         from camelot.core.view.field_attributes import \
             _sqlalchemy_to_python_type_
         return _sqlalchemy_to_python_type_[type_](field)
+
