@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -28,10 +28,13 @@ logger = logging.getLogger('camelot.admin.entity_admin')
 
 from camelot.admin.action.list_action import OpenFormView
 from camelot.admin.object_admin import ObjectAdmin
-from camelot.view.model_thread import post, model_function, gui_function
+from camelot.view.model_thread import post, model_function
+from camelot.view.utils import to_string
 from camelot.core.utils import ugettext_lazy, ugettext
 from camelot.admin.validator.entity_validator import EntityValidator
 
+from sqlalchemy import orm, exc
+
 class EntityAdmin(ObjectAdmin):
     """Admin class specific for classes that are mapped by sqlalchemy.
 This allows for much more introspection than the standard 
@@ -43,7 +46,7 @@ It has additional class attributes that customise its behaviour.
 
 .. attribute:: list_action
 
-   The :class:`camelot.admin.action.Action` that will be triggered when the
+   The :class:`camelot.admin.action.base.Action` that will be triggered when the
    user selects an item in a list of objects.  This defaults to 
    :class:`camelot.admin.action.list_action.OpenFormView`, which opens a form
    for the current object.
@@ -56,11 +59,11 @@ It has additional class attributes that customise its behaviour.
     view.  If the field named is a one2many, many2one or many2many field, the
     field name should be followed by a field name of the related entity ::
 
-        class Project(Entity):
-            oranization = OneToMany('Organization')
-            name = Field(Unicode(50))
+        class Project( Entity ):
+            oranization = OneToMany( 'Organization' )
+            name = Column( Unicode(50) )
     
-          class Admin(EntityAdmin):
+          class Admin( EntityAdmin ):
               list_display = ['organization']
               list_filter = ['organization.name']
 
@@ -71,7 +74,7 @@ It has additional class attributes that customise its behaviour.
 .. attribute:: copy_deep
 
    A dictionary of fields that will be deep copied when the user presses the copy
-   button.  This is usefull for OneToMany fields.  The key in the dictionary should
+   button.  This is useful for OneToMany fields.  The key in the dictionary should
    be the name of the field, and the value is a new dictionary that can contain other
    fields that need to be copied::
 
@@ -121,7 +124,6 @@ It has additional class attributes that customise its behaviour.
 
     def __init__(self, app_admin, entity):
         super(EntityAdmin, self).__init__(app_admin, entity)
-        from sqlalchemy import orm
         from sqlalchemy.orm.exc import UnmappedClassError
         from sqlalchemy.orm.mapper import _mapper_registry
         try:
@@ -138,8 +140,8 @@ It has additional class attributes that customise its behaviour.
         displayed in the table or the selection view.  Overwrite this method to
         change the default query, which selects all rows in the database.
         """
-        from elixir import session
-        return session.query( self.entity )
+        from camelot.core.orm import Session
+        return Session().query( self.entity )
 
     @model_function
     def get_verbose_identifier(self, obj):
@@ -187,13 +189,13 @@ It has additional class attributes that customise its behaviour.
             #
             attributes = dict(
                 python_type = str,
-                to_string = unicode,
+                to_string = to_string,
                 field_name = field_name,
                 getter = create_default_getter(field_name),
                 length = None,
                 tooltip = None,
                 background_color = None,
-                minimal_column_width = 12,
+                #minimal_column_width = 12,
                 editable = False,
                 nullable = True,
                 widget = 'str',
@@ -235,7 +237,7 @@ It has additional class attributes that customise its behaviour.
                     admin_class = forced_attributes['admin']
                     return admin_class(self.app_admin, target)
                 except KeyError:
-                    return self.get_related_entity_admin(target)
+                    return self.get_related_admin(target)
             #
             # Get the default field_attributes trough introspection if the
             # field is a mapped field
@@ -375,7 +377,6 @@ It has additional class attributes that customise its behaviour.
 
         return list(filter_generator())
 
-    @gui_function
     def create_select_view(admin, query=None, search_text=None, parent=None):
         """Returns a Qt widget that can be used to select an element from a
         query
@@ -444,7 +445,6 @@ It has additional class attributes that customise its behaviour.
         widget.update()
         return widget
 
-    @gui_function
     def create_table_view( self, gui_context ):
         """Returns a :class:`QtGui.QWidget` containing a table view
         :param gui_context: a :class:`camelot.admin.action.base.GuiContext`
@@ -452,6 +452,48 @@ It has additional class attributes that customise its behaviour.
         """
         return self.TableView( gui_context, self )
 
+    def primary_key( self, obj ):
+        """Get the primary key of an object
+        :param obj: the object to get the primary key from
+        :return: a tuple with with components of the primary key, or none
+            if the object has no primary key yet or any more.
+        """
+        if not self.is_persistent( obj ):
+            return None
+        return self.mapper.primary_key_from_instance( obj )
+    
+    def get_modifications( self, obj ):
+        """Get the modifications on an object since the last flush.
+        :param obj: the object for which to get the modifications
+        :return: a dictionary with the changed attributes and their old
+           value
+        """
+        from sqlalchemy import orm
+        from sqlalchemy.orm.exc import UnmappedClassError
+        state = orm.attributes.instance_state( obj )
+        dict_ = state.dict
+        modifications = dict()
+        for attr in state.manager.attributes:
+            if not hasattr( attr.impl, 'get_history' ):
+                continue
+            (added, unchanged, deleted) = \
+                    attr.impl.get_history( state, dict_ )
+            if added or deleted:
+                old_value = None
+                if deleted:
+                    old_value = deleted[0]
+                    #
+                    # in case of relations, get the primary key of the object
+                    # instead of the object itself
+                    #
+                    try:
+                        mapper = orm.class_mapper( type( old_value ) )
+                        old_value = mapper.primary_key_from_instance( old_value )
+                    except UnmappedClassError:
+                        pass
+                modifications[ attr.key ] = old_value
+        return modifications
+        
     @model_function
     def delete(self, entity_instance):
         """Delete an entity instance"""
@@ -478,14 +520,15 @@ It has additional class attributes that customise its behaviour.
                 if not None in primary_key and len(primary_key)==1:
                     pk = primary_key[0]
                     # save the state before the update
-                    from camelot.model.memento import BeforeDelete
+                    from camelot.model.memento import Memento
                     # only register the delete when the camelot model is active
-                    if hasattr(BeforeDelete, 'query'):
-                        from camelot.model.authentication import getCurrentAuthentication
-                        history = BeforeDelete( model = unicode( self.entity.__name__ ),
-                                                primary_key = pk,
-                                                previous_attributes = {},
-                                                authentication = getCurrentAuthentication() )
+                    if hasattr(Memento, 'query'):
+                        from camelot.model.authentication import get_current_authentication
+                        history = Memento( model = unicode( self.entity.__name__ ),
+                                           memento_type = 'before_delete',
+                                           primary_key = pk,
+                                           previous_attributes = {},
+                                           authentication = get_current_authentication() )
                 entity_instance.delete()
                 session.flush( [entity_instance] )
                 if history:
@@ -505,7 +548,33 @@ It has additional class attributes that customise its behaviour.
         from sqlalchemy.orm.session import Session
         session = Session.object_session( entity_instance )
         if session:
+            modifications = {}
+            try:
+                modifications = self.get_modifications( entity_instance )
+            except Exception, e:
+                # todo : there seems to be a bug in sqlalchemy that causes the
+                #        get history to fail in some cases
+                logger.error( 'could not get modifications from object', exc_info = e )
             session.flush( [entity_instance] )
+            #
+            # If needed, track the changes
+            #
+            primary_key = self.primary_key( entity_instance )
+            if modifications and (primary_key != None) and len(primary_key)==1:
+                from camelot.model.memento import Memento
+                # only register the update when the camelot model is active
+                if hasattr(Memento, 'query'):
+                    from camelot.model.authentication import get_current_authentication
+                    history = Memento( model = unicode( self.entity.__name__ ),
+                                       memento_type = 'before_update',
+                                       primary_key = primary_key[0],
+                                       previous_attributes = modifications,
+                                       authentication = get_current_authentication() )
+
+                    try:
+                        history.flush()
+                    except exc.DatabaseError, e:
+                        self.logger.error( 'Programming Error, could not flush history', exc_info = e )
 
     @model_function
     def refresh(self, entity_instance):
@@ -547,7 +616,7 @@ It has additional class attributes that customise its behaviour.
         :return: a list of tuples of type [(field_name, field_attributes)]
         """
         if self.expanded_list_search == None:
-            field_list = self.list_display
+            field_list = self.get_table().get_fields()
         else:
             field_list = self.expanded_list_search
         return [(field, self.get_field_attributes(field))
@@ -573,7 +642,6 @@ It has additional class attributes that customise its behaviour.
         #
         # serialize the object to be copied
         #
-        # @todo: this code depends on elixir
         serialized = obj.to_dict(deep=self.copy_deep, exclude=[c.name for c in self.mapper.primary_key]+self.copy_exclude)
         #
         # make sure we don't move duplicated OneToMany relations from the
@@ -605,3 +673,4 @@ It has additional class attributes that customise its behaviour.
                              property.key,
                              getattr( obj, property.key ) )
         return new_obj
+
