@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -28,10 +28,10 @@ import logging
 logger = logging.getLogger('camelot.view.object_admin')
 
 from camelot.admin.action.form_action import CloseForm
-from camelot.view.model_thread import gui_function, model_function
+from camelot.view.model_thread import model_function
 from camelot.view.controls.tableview import TableView
-from camelot.core.utils import ugettext as _
-from camelot.core.utils import ugettext_lazy
+from camelot.view.utils import to_string
+from camelot.core.utils import ugettext_lazy, ugettext as _
 from camelot.view.proxy.collection_proxy import CollectionProxy
 from validator.object_validator import ObjectValidator
 from PyQt4 import QtCore
@@ -157,18 +157,23 @@ be specified using the verbose_name attribute.
 
 .. attribute:: form_actions
 
-    Actions to be accessible by pushbuttons on the side of a form,
-    a list of tuples (button_label, action_function) where action_function
-    takes as its single argument, a method that returns the the object that
-    was displayed by the form when the button was pressed::
+    Actions to be accessible by pushbuttons on the side of a form, a list of :class:`camelot.admin.action.base.Action` objects. ::
 
-        class Admin(EntityAdmin):
-            form_actions = [('Foo', lamda o_getter:print 'foo')]
+        class Admin( EntityAdmin ):
+            form_actions = [CloseForm()]
+            
+    These actions will be triggered with a :class:`camelot.admin.action.form_action.FormActionModelContext` as the `model_context` parameter
+    in the :meth:`camelot.admin.action.base.Action.model_run` method.
             
 .. attribute:: related_toolbar_actions
 
-    list of actions that appear in the toolbar of a OneToMany editor.
+    list of actions that appear in the toolbar of a `OneToMany` editor.
 
+.. attribute:: drop_action
+
+    the action that is triggered when a drag and drop occured on the table
+    view 
+    
 **Field attributes**
 
 .. attribute:: field_attributes
@@ -177,12 +182,13 @@ be specified using the verbose_name attribute.
     attributes on how they should be displayed.  All of these attributes
     are propagated to the constructor of the delegate of this field::
 
-        class Movie(Entity):
-            title = Field(Unicode(50))
+        class Movie( Entity ):
+        
+            title = Column( Unicode(50) )
     
-            class Admin(EntityAdmin):
+            class Admin( EntityAdmin ):
                 list_display = ['title']
-                field_attributes = dict(title=dict(editable=False))
+                field_attributes = { 'title' : {'editable':False} }
 
     The :ref:`doc-admin-field_attributes` documentation describes the various keys
     that can be used in the field attributes class attribute of an ObjectAdmin 
@@ -192,22 +198,23 @@ be specified using the verbose_name attribute.
 
 .. attribute:: form_state
 
-    Set this attribute to 'maximized' or 'minimized' for respective behaviour. These are the only two defined at the moment.
-    Please use the constants defined in camelot.core.constants (MINIMIZE and MAXIMIZE).
-    Note that this attr needs to be set at the form, highest in the form hierarchy to work. Setting this on embedded forms
-    will not influence the window state. Example::
+    Set this attribute to `maximized` or `minimized` for respective behaviour ::
 
-        class Movie(Entity):
-            title = Field(Unicode(50))
+        class Movie( Entity ):
+        
+            title = Column( Unicode(50) )
     
-            class Admin(EntityAdmin):
-                from camelot.core import constants
+            class Admin( EntityAdmin ):
                 list_display = ['title']
-                form_state = constants.MAXIMIZED
-                field_attributes = dict(title=dict(editable=False))
+                form_state = 'maximized'
 
 **Varia**
 
+.. attribute:: name
+
+    The name of the group in settings in which user dependent settings will
+    be stored, defaults to the class name for which this Admin class is used.
+    
 .. attribute:: model
 
     The QAbstractItemModel class to be used to display collections of this object,
@@ -218,6 +225,7 @@ be specified using the verbose_name attribute.
     The QWidget class to be used when a table view is needed
     """
     
+    name = None
     verbose_name = None
     verbose_name_plural = None
     list_display = []
@@ -234,35 +242,33 @@ be specified using the verbose_name attribute.
     list_size = (600, 600)
     form_size = (700, 500)
     form_actions = []
+    related_toolbar_actions = []
     field_attributes = {}
     form_state = None
     icon = None # Default
     #
     # Behavioral attributes
     # 
+    drop_action = None
     save_mode = 'on_edit'
     delete_mode = 'on_request'
 
     TableView = TableView
 
-    def __init__(self, app_admin, entity):
+    def __init__( self, app_admin, entity ):
         """
         :param app_admin: the application admin object for this application, 
             if None, then the default application_admin is taken
         :param entity: the entity class for which this admin instance is to be
             used
         """
-        from camelot.view.remote_signals import get_signal_handler
         if not app_admin:
             from camelot.admin.application_admin import get_application_admin
             self.app_admin = get_application_admin()
         else:
             self.app_admin = app_admin
-        self.rsh = get_signal_handler()
         if entity:
-            from camelot.view.model_thread import get_model_thread
             self.entity = entity
-            self.mt = get_model_thread()
         #
         # caches to prevent recalculation of things
         #
@@ -276,7 +282,13 @@ be specified using the verbose_name attribute.
         return 'ObjectAdmin(%s)' % str(self.entity.__name__)
 
     def get_name(self):
-        return self.get_verbose_name()
+        """ The name of the group in settings in which user dependent settings 
+        will be stored, this is either the `name` attribute of this class or, 
+        the class name of the class for which this Admin class is used.
+        
+        :return: a string with the name of the settings group        
+        """
+        return self.name or self.entity.__name__
 
     def get_verbose_name(self):
         
@@ -313,6 +325,16 @@ be specified using the verbose_name attribute.
 
     def get_save_mode(self):
         return self.save_mode
+    
+    def get_settings( self ):
+        """A settings object in which settings related to this admin can be
+        stored.
+        
+        :return: a :class:`QtCore.QSettings` object
+        """
+        settings = self.app_admin.get_settings()
+        settings.beginGroup( self.get_name()[:255] )
+        return settings
 
     def get_delete_mode(self):
         return self.delete_mode
@@ -331,6 +353,21 @@ be specified using the verbose_name attribute.
         app_admin = self.get_application_admin()
         from camelot.admin.action.form_action import structure_to_form_actions
         return app_admin.get_form_actions() + structure_to_form_actions( self.form_actions )
+    
+    @model_function
+    def get_form_toolbar_actions( self, toolbar_area ):
+        """
+        By default this function will return the same as :meth:`camelot.admin.application_admin.ApplicationAdmin.get_form_toolbar_actions`
+        
+        :param toolbar_area: an instance of :class:`Qt.ToolBarArea` indicating
+            where the toolbar actions will be positioned
+            
+        :return: a list of :class:`camelot.admin.action.base.Action` objects
+            that should be displayed on the toolbar of a form view.  return
+            None if no toolbar should be created.
+        """        
+        app_admin = self.get_application_admin()
+        return app_admin.get_form_toolbar_actions( toolbar_area )
 
     def get_related_toolbar_actions( self, toolbar_area, direction ):
         """Specify the toolbar actions that should appear in a OneToMany editor.
@@ -342,7 +379,8 @@ be specified using the verbose_name attribute.
         :return: a list of :class:`camelot.admin.action.base.Action` objects
         """
         app_admin = self.get_application_admin()
-        return app_admin.get_related_toolbar_actions( toolbar_area, direction )
+        return self.related_toolbar_actions or \
+               app_admin.get_related_toolbar_actions( toolbar_area, direction )
     
     @model_function
     def get_list_actions(self):
@@ -368,7 +406,7 @@ be specified using the verbose_name attribute.
         """
         subclasses = []
         for subclass in self.entity.__subclasses__():
-            subclass_admin = self.get_related_entity_admin(subclass)
+            subclass_admin = self.get_related_admin( subclass )
             if subclass_admin!=self:
                 subclasses.append((
                     subclass_admin,
@@ -395,10 +433,6 @@ be specified using the verbose_name attribute.
             logger.warn('no related admin found for %s' % (cls.__name__))
         return related_admin
         
-    def get_related_entity_admin(self, entity):
-        """deprecated : use get_related_admin"""
-        return self.get_related_admin(entity)
-
     def get_static_field_attributes(self, field_names):
         """
         Convenience function to get all the field attributes
@@ -495,12 +529,13 @@ be specified using the verbose_name attribute.
             #
             attributes = dict(
                 getter=create_default_getter(field_name),
+                to_string = to_string,
                 field_name=field_name,
                 python_type=str,
                 length=None,
                 tooltip=None,
                 background_color=None,
-                minimal_column_width=12,
+                #minimal_column_width=12,
                 editable=False,
                 nullable=True,
                 widget='str',
@@ -545,7 +580,7 @@ be specified using the verbose_name attribute.
                     admin_class = fa['admin']
                     return admin_class(self.app_admin, target)
                 except KeyError:
-                    return self.get_related_entity_admin(target)
+                    return self.get_related_admin(target)
 
             if 'target' in attributes:
                 attributes['admin'] = get_entity_admin(attributes['target'])
@@ -553,6 +588,14 @@ be specified using the verbose_name attribute.
             self._field_attributes[field_name] = attributes
             return attributes
 
+    def get_table( self ):
+        """The definition of the table to be used in a list view
+        :return: a `camelot.admin.table.Table` object
+        """
+        from camelot.admin.table import structure_to_table
+        table = structure_to_table( self.list_display )
+        return table
+    
     @model_function
     def get_columns(self):
         """
@@ -568,8 +611,9 @@ be specified using the verbose_name attribute.
                    'name':'Field name'}),
                  ...]
         """
+        table = self.get_table()
         return [(field, self.get_field_attributes(field))
-                for field in self.list_display]
+                for field in table.get_fields() ]
 
     def get_validator( self ):
         """Get a validator object
@@ -586,10 +630,8 @@ be specified using the verbose_name attribute.
     def get_fields(self):
         if self.form_display:
             fields = self.get_form_display().get_fields()
-        elif self.fields:
-            fields = self.fields
         else:
-            fields = self.list_display
+            fields = self.get_table().get_fields()
         fields_and_attributes =  [
                 (field, self.get_field_attributes(field))
                 for field in fields
@@ -620,7 +662,7 @@ be specified using the verbose_name attribute.
         if self.form_display:
             return structure_to_form(self.form_display)
         if self.list_display:
-            return Form(self.list_display)
+            return Form( self.get_table().get_fields() )
         return Form([])
 
     def _apply_form_state(self, widget):
@@ -633,7 +675,6 @@ be specified using the verbose_name attribute.
             if self.form_state == constants.MINIMIZED:
                 widget.setWindowState(QtCore.Qt.WindowMinimized)
         
-    @gui_function
     def create_form_view(self, title, model, index, parent=None):
         """Creates a Qt widget containing a form view, for a specific index in
         a model.  Use this method to create a form view for a collection of objects,
@@ -710,7 +751,6 @@ be specified using the verbose_name attribute.
                         exc_info=exc
                     )
 
-    @gui_function
     def create_object_form_view(self, title, object_getter, parent=None):
         """Create a form view for a single object, :kbd:`PgUp`/:kbd:`PgDown` 
         will do nothing.
@@ -743,7 +783,6 @@ be specified using the verbose_name attribute.
                             self.get_fields )
         return self.create_form_view(title, model, 0, parent)
 
-    @gui_function
     def create_new_view(admin, related_collection_proxy=None, parent=None):
         """Create a Qt widget containing a form to create a new instance of the
         entity related to this admin class
@@ -844,6 +883,22 @@ be specified using the verbose_name attribute.
             form.setMinimumSize(admin.form_size[0], admin.form_size[1])
         return form
 
+    def primary_key( self, obj ):
+        """Get the primary key of an object
+        :param obj: the object to get the primary key from
+        :return: a tuple with with components of the primary key, or an
+            emtpy list if the object has no primary key yet or any more.
+        """
+        return []
+    
+    def get_modifications( self, obj ):
+        """Get the modifications on an object since the last flush.
+        :param obj: the object for which to get the modifications
+        :return: a dictionary with the changed attributes and their old
+           value
+        """
+        return dict()
+    
     @model_function
     def delete(self, entity_instance):
         """Delete an entity instance"""
@@ -887,4 +942,3 @@ be specified using the verbose_name attribute.
         """Duplicate this entity instance"""
         new_entity_instance = entity_instance.__class__()
         return new_entity_instance
-
