@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -24,39 +24,7 @@
 
 # begin session setup
 import elixir
+from camelot.core.orm import Session
 
-from sqlalchemy import MetaData
-
-from sqlalchemy.orm import scoped_session, create_session
-
-elixir.session = scoped_session( create_session )
-
-from camelot.core.conf import settings
-
-metadata = MetaData()
-
-__metadata__ = metadata
-
-__metadata__.bind = settings.ENGINE()
-__metadata__.autoflush = False
-__metadata__.transactional = False
-
+elixir.session = Session
 # end session setup
-
-import authentication
-import fixture
-import i18n
-import memento
-import synchronization
-import type_and_status
-import batch_job
-
-# dummy variable to prevent pycheckers warnings on unused imports
-__model__ = [authentication, 
-             fixture, 
-             i18n, 
-             memento, 
-             synchronization, 
-             type_and_status,
-             batch_job]
-
