@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -37,7 +37,7 @@ from elixir.fields import Field
 from elixir.relationships import ManyToOne
 
 from camelot.core.utils import ugettext_lazy as _
-from camelot.model import metadata
+from camelot.core.sql import metadata
 from camelot.view import filters
 from camelot.admin.entity_admin import EntityAdmin
 from camelot.admin.action import Action
@@ -139,3 +139,4 @@ class BatchJob(Entity):
         
 BatchJob = documented_entity()( BatchJob )
 
+
