@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -24,6 +24,9 @@
 from floatdelegate import FloatDelegate
 
 class CurrencyDelegate(FloatDelegate):
-    """Custom delegate for currency values"""
+    """Custom delegate for currency values
+    """
     pass
 
+
+
