@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -48,9 +48,11 @@ class DateDelegate(CustomDelegate):
         self.drawBackground(painter, option, index)
         value = variant_to_pyobject( index.model().data( index, Qt.EditRole ) )
         
-        value_str = u'0/0/0'
+        value_str = u''
         if value not in (None, ValueLoading):
             value_str = QtCore.QDate(value).toString(self.date_format)
             
         self.paint_text(painter, option, index, value_str, horizontal_align=Qt.AlignRight )
         painter.restore()
+
+
