@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -30,8 +30,9 @@ from camelot.core.utils import variant_to_pyobject
 from camelot.view.proxy import ValueLoading
 
 class FileDelegate(CustomDelegate):
-    """Delegate for camelot.types.file fields.  Expects values of type camelot.core.files.storage.StoredFile.
-  """
+    """Delegate for :class:`camelot.types.File` columns.  Expects values of type 
+    :class:`camelot.core.files.storage.StoredFile`.
+    """
     
     __metaclass__ = DocumentationMetaclass
     
@@ -60,3 +61,5 @@ class FileDelegate(CustomDelegate):
             
         painter.restore()
 
+
+
