@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -29,7 +29,9 @@ from PyQt4 import QtGui, QtCore
 from PyQt4.QtCore import Qt
 
 class ImageDelegate(FileDelegate):
-    """
+    """Delegate for :class:`camelot.types.Image` fields.  Expects values of type 
+    :class:`camelot.core.files.storage.StoredImage`.
+
     .. image:: /_static/image.png
     """
     
@@ -58,3 +60,4 @@ class ImageDelegate(FileDelegate):
                 painter.setPen(pen)
                 painter.drawRect(rect)
         painter.restore()
+
