@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -25,5 +25,10 @@ from plaintextdelegate import PlainTextDelegate
 from camelot.view.controls.editors import LanguageEditor
 
 class LanguageDelegate( PlainTextDelegate ):
+    """Delegate for :class:`camelot.types.Language` columns.  Expects string
+    values.
+    """
     
     editor = LanguageEditor
+
+
