@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -26,6 +26,8 @@ from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import Qt
 
+from camelot.view.model_thread import object_thread
+
 from customeditor import CustomEditor, set_background_color_palette, draw_tooltip_visualization
 import re
 
@@ -66,8 +68,8 @@ class CodeEditor(CustomEditor):
     def __init__(self, parent=None, parts=['99','AA'], editable=True, field_name='code', **kwargs):
         CustomEditor.__init__(self, parent)
         self.setObjectName( field_name )
+        self.value = None
         self.parts = parts
-        self.part_editors = []
         layout = QtGui.QHBoxLayout()
         layout.setContentsMargins( 0, 0, 0, 0)
         layout.setSpacing(0)
@@ -83,7 +85,7 @@ class CodeEditor(CustomEditor):
                 editor.setEnabled(False)
             space_width = editor.fontMetrics().size(Qt.TextSingleLine, 'A').width()
             editor.setMaximumWidth(space_width*(part_length+1))
-            self.part_editors.append(editor)
+            editor.setObjectName( 'part_%i'%i )
             layout.addWidget(editor)
             editor.editingFinished.connect(self.emit_editing_finished)
         self.setLayout(layout)
@@ -94,39 +96,48 @@ class CodeEditor(CustomEditor):
         # signals are generated by the QLineEdit itself, maybe because of the
         # pattern.  this can lead to spurious data entry in a form, therefor this
         # check is necessary
-        for editor in self.part_editors:
+        for editor in self._get_part_editors():
             if editor.isModified():
                 self.editingFinished.emit()
                 return
 
+    def _get_part_editors( self ):
+        for i in range( len( self.parts ) ):
+            part_editor = self.findChild( QtGui.QWidget,
+                                          'part_%s'%i )
+            yield part_editor
+            
     def set_enabled(self, editable=True):
-        for editor in self.part_editors:
+        for editor in self._get_part_editors():
             editor.setEnabled(editable)
 
     def set_value(self, value):
+        assert object_thread( self )
         value = CustomEditor.set_value(self, value)
         if value:
             old_value = self.get_value()
             if value!=old_value:
-                for part_editor, part in zip(self.part_editors, value):
+                for part_editor, part in zip( self._get_part_editors(), value ):
                     part_editor.setText(unicode(part))
         else:
-            for part_editor in self.part_editors:
+            for part_editor in self._get_part_editors():
                 part_editor.setText('')
 
     def get_value(self):
+        assert object_thread( self )
         value = []
-        for part in self.part_editors:
-            value.append(unicode(part.text()))
+        for part_editor in self._get_part_editors():
+            value.append( unicode( part_editor.text() ) )
         return CustomEditor.get_value(self) or value
 
     def set_background_color(self, background_color):
-        for editor in self.part_editors:
+        for editor in self._get_part_editors():
             set_background_color_palette( editor, background_color )
             
     def set_field_attributes(self, editable = True,
                                    background_color = None,
                                    tooltip = None, **kwargs):
+        assert object_thread( self )
         self.set_enabled(editable)
         self.set_background_color(background_color)
         self.layout().itemAt(0).widget().setToolTip(unicode(tooltip or ''))
