@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -206,14 +206,21 @@ class FileEditor(CustomEditor):
             self.stored_file_ready,
             filter=self.filter,
             remove_original=self.remove_original,
-        )        
+        )
+        
     def open_button_clicked(self):
         from camelot.view.storage import open_stored_file
         open_stored_file(self, self.value)
 
     def clear_button_clicked(self):
-        self.value = None
-        self.editingFinished.emit()
+        answer = QtGui.QMessageBox.question( self, 
+                                             _('Remove this file ?'), 
+                                             _('If you continue, you will no longer be able to open this file.'), 
+                                             QtGui.QMessageBox.Yes,
+                                             QtGui.QMessageBox.No )
+        if answer == QtGui.QMessageBox.Yes:
+            self.value = None
+            self.editingFinished.emit()
 
     #
     # Drag & Drop
@@ -238,3 +245,4 @@ class FileEditor(CustomEditor):
                     remove_original=self.remove_original,
                     filename = filename,
                 )
+
