@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -49,4 +49,7 @@ class OneToManyChoicesEditor(ChoicesEditor):
         """Makes sure choices are not reset when changing the
         field attributes"""
         self.setEnabled(editable!=False)
-
+        
+    def set_value(self, value):
+        # post to make sure the set value occurs after the set choices
+        post( lambda:value, super( OneToManyChoicesEditor, self ).set_value )
