@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -33,13 +33,13 @@ from PyQt4.QtCore import Qt
 from PyQt4.QtGui import QSizePolicy
 
 from camelot.admin.action.list_action import ListActionGuiContext
-from camelot.core.utils import ugettext as _
+from camelot.core.utils import variant_to_pyobject, ugettext as _
 from camelot.view.proxy.queryproxy import QueryTableProxy
 from camelot.view.controls.view import AbstractView
 from camelot.view.controls.user_translatable_label import UserTranslatableLabel
 from camelot.view.controls.progress_dialog import ProgressDialog
 from camelot.view.model_thread import post
-from camelot.view.model_thread import gui_function
+from camelot.view.model_thread import object_thread
 from camelot.view.model_thread import model_function
 from camelot.view import register
 
@@ -51,6 +51,7 @@ class FrozenTableWidget( QtGui.QTableView ):
 
     def __init__(self, parent, columns_frozen):
         super(FrozenTableWidget, self).__init__(parent)
+        assert object_thread( self )
         self.setSelectionBehavior( QtGui.QAbstractItemView.SelectRows )
         self.setEditTriggers( QtGui.QAbstractItemView.SelectedClicked |
                               QtGui.QAbstractItemView.DoubleClicked |
@@ -61,11 +62,62 @@ class FrozenTableWidget( QtGui.QTableView ):
     def currentChanged(self, current, previous):
         """When the current index has changed, prevent it to jump to
         a column that is not frozen"""
+        assert object_thread( self )
         if current.column() >= self._columns_frozen:
             current = self.model().index( current.row(), -1 )
         if previous.column() >= self._columns_frozen:
             previous = self.model().index( previous.row(), -1 )
         super(FrozenTableWidget, self).currentChanged(current, previous)
+        
+class ColumnGroupsWidget( QtGui.QTabBar ):
+    """A tabbar the user can use to select a group of columns within an
+    item view.
+    
+    :param table: a :class:`camelot.admin.table.Table` object, describing the 
+       column groups.
+    :param table_widget: a :class:`QtGui.QTableView` widget of which columns will
+       be hidden and shown depending on the selected tab.
+    :param parent: a :class:`QtGui.QWidget`
+    """
+    
+    def __init__( self, table, table_widget, parent = None ):
+        from camelot.admin.table import ColumnGroup
+        super( ColumnGroupsWidget, self ).__init__( parent )
+        assert object_thread( self )
+        self.setShape( QtGui.QTabBar.RoundedSouth )
+        self.groups = dict()
+        self.table_widget = table_widget
+        column_index = 0
+        tab_index = 0
+        for column in table.columns:
+            if isinstance( column, ColumnGroup ):
+                self.addTab( unicode( column.verbose_name ) )
+                previous_column_index = column_index
+                column_index = column_index + len( column.get_fields() )
+                self.groups[ tab_index ] = ( previous_column_index,
+                                             column_index )
+                tab_index += 1
+            else:
+                column_index += 1
+        self.currentChanged.connect( self._current_index_changed )
+        
+    @QtCore.pyqtSlot( QtCore.QModelIndex, int, int )
+    def columns_changed( self, index, first_column, last_column ):
+        assert object_thread( self )
+        self._current_index_changed( self.currentIndex() )
+        
+    @QtCore.pyqtSlot()
+    def model_reset( self ):
+        assert object_thread( self )
+        self._current_index_changed( self.currentIndex() )
+        
+    @QtCore.pyqtSlot( int )
+    def _current_index_changed( self, current_index ):
+        assert object_thread( self )
+        for tab_index, (first_column, last_column) in self.groups.items():
+            for column_index in range( first_column, last_column ):
+                self.table_widget.setColumnHidden( column_index,
+                                                   tab_index != current_index )
 
 class TableWidget( QtGui.QTableView ):
     """A widget displaying a table, to be used within a TableView.  This is a
@@ -82,7 +134,6 @@ and above the text.
 """
 
     margin = 5
-    
     keyboard_selection_signal = QtCore.pyqtSignal()
 
     def __init__( self, parent = None, columns_frozen = 0, lines_per_row = 1 ):
@@ -92,7 +143,9 @@ and above the text.
         """
         QtGui.QTableView.__init__( self, parent )
         logger.debug( 'create TableWidget' )
+        assert object_thread( self )
         self._columns_frozen = columns_frozen
+        self._columns_changed = dict()
         self.setSelectionBehavior( QtGui.QAbstractItemView.SelectRows )
         self.setEditTriggers( QtGui.QAbstractItemView.SelectedClicked |
                               QtGui.QAbstractItemView.DoubleClicked |
@@ -108,6 +161,7 @@ and above the text.
         self.setVerticalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
         self.horizontalHeader().sectionClicked.connect(
             self.horizontal_section_clicked )
+        self.horizontalHeader().sectionResized.connect( self._save_section_width )
         if columns_frozen:
             frozen_table_view = FrozenTableWidget(self, columns_frozen)
             frozen_table_view.setObjectName( 'frozen_table_view' )
@@ -128,29 +182,70 @@ and above the text.
             frozen_table_view.setVerticalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
 
     @QtCore.pyqtSlot(int, int, int)
-    def _update_section_width(self, logical_index, _int, new_size):
+    def _update_section_width(self, logical_index, _old_size, new_size):
+        assert object_thread( self )
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
-        if logical_index<self._columns_frozen and frozen_table_view:
+        if logical_index < self._columns_frozen and frozen_table_view:
             frozen_table_view.setColumnWidth( logical_index, new_size)
             self._update_frozen_table()
+            
+    def timerEvent( self, event ):
+        """On timer event, save changed column widths to the model
+        """
+        assert object_thread( self )
+        for logical_index, new_width in self._columns_changed.items():
+            if self.horizontalHeader().isSectionHidden( logical_index ):
+                # don't save the width of a hidden section, since this will
+                # result in setting the width to 0
+                continue
+            old_size = variant_to_pyobject( self.model().headerData( logical_index, 
+                                                                     Qt.Horizontal, 
+                                                                     Qt.SizeHintRole ) )
+            # when the size is different from the one from the model, the
+            # user changed it
+            if old_size.width() != new_width:
+                new_size = QtCore.QSize( new_width, old_size.height() )
+                self.model().setHeaderData( logical_index, 
+                                            Qt.Horizontal,
+                                            new_size,
+                                            Qt.SizeHintRole )
+        self._columns_changed = dict()
+        super( TableWidget, self ).timerEvent( event )
+        
+    @QtCore.pyqtSlot(int, int, int)
+    def _save_section_width(self, logical_index, _old_size, new_width ):
+        # instead of storing the width immediately, a timer is started to store
+        # the width when all event processing is done.  because at this time
+        # we cannot yet determine if the section at logical_index is hidden
+        # or not
+        #
+        # there is no need to start the timer, since this is done by the 
+        # QAbstractItemView itself for doing the layout, here we only store
+        # which column needs to be saved.
+        assert object_thread( self )
+        self._columns_changed[ logical_index ] = new_width
 
     @QtCore.pyqtSlot(int, int, int)
     def _update_section_height(self, logical_index, _int, new_size):
+        assert object_thread( self )
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
         if frozen_table_view:
             frozen_table_view.setRowHeight(logical_index, new_size)
 
     def setItemDelegate(self, item_delegate):
+        assert object_thread( self )
         super(TableWidget, self).setItemDelegate(item_delegate)
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
         if frozen_table_view:
             frozen_table_view.setItemDelegate(item_delegate)
 
     def resizeEvent(self, event):
+        assert object_thread( self )
         super(TableWidget, self).resizeEvent(event)
         self._update_frozen_table()
 
     def moveCursor(self, cursorAction, modifiers):
+        assert object_thread( self )
         current = super(TableWidget, self).moveCursor(cursorAction, modifiers)
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
         if frozen_table_view:
@@ -165,10 +260,12 @@ and above the text.
         return current
 
     def scrollTo(self, index, hint):
+        assert object_thread( self )
         if(index.column()>=self._columns_frozen):
             super(TableWidget, self).scrollTo(index, hint)
 
     def edit(self, index, trigger=None, event=None):
+        assert object_thread( self )
         #
         # columns in the frozen part should never be edited, because this might result
         # in an editor opening below the frozen column that contains the old value
@@ -182,6 +279,7 @@ and above the text.
 
     @QtCore.pyqtSlot()
     def _update_frozen_table(self):
+        assert object_thread( self )
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
         if frozen_table_view:
             selection_model = self.selectionModel()
@@ -206,6 +304,7 @@ and above the text.
     @QtCore.pyqtSlot( int )
     def horizontal_section_clicked( self, logical_index ):
         """Update the sorting of the model and the header"""
+        assert object_thread( self )
         header = self.horizontalHeader()
         order = Qt.AscendingOrder
         if not header.isSortIndicatorShown():
@@ -224,6 +323,7 @@ and above the text.
         
         those assertion failures only exist in QT debug builds.
         """
+        assert object_thread( self )
         current_index = self.currentIndex()
         if not current_index:
             return
@@ -235,6 +335,7 @@ and above the text.
             table_widget.closePersistentEditor( current_index )
                 
     def setModel( self, model ):
+        assert object_thread( self )
         #
         # An editor might be open that is no longer available for the new
         # model.  Not closing this editor, results in assertion failures
@@ -255,6 +356,7 @@ and above the text.
         
     @QtCore.pyqtSlot(QtCore.QModelIndex, QtCore.QModelIndex)
     def activated( self, selectedIndex, previousSelectedIndex ):
+        assert object_thread( self )
         option = QtGui.QStyleOptionViewItem()
         new_size = self.itemDelegate( selectedIndex ).sizeHint( option,
                                                                 selectedIndex )
@@ -266,23 +368,56 @@ and above the text.
                                      self._minimal_row_height ) )
                                      
     def keyPressEvent(self, e):
+        assert object_thread( self )
         if self.hasFocus() and e.key() in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return):
             self.keyboard_selection_signal.emit()
         else:
             super(TableWidget, self).keyPressEvent(e) 
 
-class AdminTableWidget(TableWidget):
+class AdminTableWidget( QtGui.QWidget ):
     """A table widget that inspects the admin class and changes the behavior
     of the table as specified in the admin class"""
     
     def __init__(self, admin, parent=None):
+        super( AdminTableWidget, self ).__init__( parent )
+        assert object_thread( self )
         self._admin = admin
-        super(AdminTableWidget, self).__init__( columns_frozen = admin.list_columns_frozen,
-                                                lines_per_row = admin.lines_per_row,
-                                                parent=parent )
-                                   
+        table_widget = TableWidget( columns_frozen = admin.list_columns_frozen,
+                                    lines_per_row = admin.lines_per_row,
+                                    parent = self )
+        table_widget.setObjectName( 'table_widget' )
+        column_groups = ColumnGroupsWidget( admin.get_table(), table_widget )
+        column_groups.setObjectName( 'column_groups' )
+        layout = QtGui.QVBoxLayout()
+        layout.setSpacing( 0 )
+        layout.setMargin( 0 )
+        layout.addWidget( table_widget )
+        layout.addWidget( column_groups )
+        self.setLayout( layout )
+        if admin.drop_action != None:
+            table_widget.setDragDropMode( QtGui.QAbstractItemView.DragDrop )
+            table_widget.setDropIndicatorShown( True )
+                        
+    def __getattr__( self, name ):
+        table_widget = self.findChild( QtGui.QWidget, 'table_widget' )
+        if table_widget != None:
+            return getattr( table_widget, name )
+        
+    def setModel( self, model ):
+        assert object_thread( self )
+        table_widget = self.findChild( QtGui.QWidget, 'table_widget' )
+        column_groups = self.findChild( QtGui.QWidget, 'column_groups' )
+        if table_widget != None:
+            model.columnsInserted.connect( column_groups.columns_changed )
+            model.columnsRemoved.connect( column_groups.columns_changed )
+            model.layoutChanged.connect( column_groups.model_reset )
+            model.modelReset.connect( column_groups.model_reset )
+            table_widget.setModel( model )
+            column_groups.model_reset()
+            
     @QtCore.pyqtSlot()
     def delete_selected_rows(self):
+        assert object_thread( self )
         logger.debug( 'delete selected rows called' )
         confirmed = True
         rows = set( index.row() for index in self.selectedIndexes() )
@@ -309,7 +444,8 @@ class AdminTableWidget(TableWidget):
             progress_dialog.exec_()
 
     @QtCore.pyqtSlot()
-    def copy_selected_rows(self):            
+    def copy_selected_rows(self):
+        assert object_thread( self )
         for row in set( map( lambda x: x.row(), self.selectedIndexes() ) ):
             self.model().copy_row( row )
         
@@ -321,9 +457,11 @@ class RowsWidget( QtGui.QLabel ):
 
     def __init__( self, parent ):
         QtGui.QLabel.__init__( self, parent )
+        assert object_thread( self )
         self.setFont( self._number_of_rows_font )
 
     def setNumberOfRows( self, rows ):
+        assert object_thread( self )
         self.setText( _('(%i rows)')%rows )
 
 class HeaderWidget( QtGui.QWidget ):
@@ -340,6 +478,7 @@ class HeaderWidget( QtGui.QWidget ):
 
     def __init__( self, parent, admin ):
         QtGui.QWidget.__init__( self, parent )
+        assert object_thread( self )
         self._admin = admin
         layout = QtGui.QVBoxLayout()
         widget_layout = QtGui.QHBoxLayout()
@@ -356,11 +495,11 @@ class HeaderWidget( QtGui.QWidget ):
             widget_layout.addWidget( self.number_of_rows )
         else:
             self.number_of_rows = None
-        layout.addLayout( widget_layout )
+        layout.addLayout( widget_layout, 0 )
         self._expanded_filters_created = False
         self._expanded_search = QtGui.QWidget()
         self._expanded_search.hide()
-        layout.addWidget(self._expanded_search)
+        layout.addWidget( self._expanded_search, 1 )
         self.setLayout( layout )
         self.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )
         self.setNumberOfRows( 0 )
@@ -371,8 +510,10 @@ class HeaderWidget( QtGui.QWidget ):
         with more options to filter rows in the table
         :param columns: a list of tuples with field names and attributes
         """
+        assert object_thread( self )
         from camelot.view.controls.filter_operator import FilterOperator
-        layout = QtGui.QHBoxLayout()
+        from camelot.view.flowlayout import FlowLayout
+        layout = FlowLayout()
         layout.setSpacing( 2 )
         layout.setContentsMargins( 0, 0, 0, 0 )
         for i, (field, attributes) in enumerate(columns):
@@ -387,11 +528,12 @@ class HeaderWidget( QtGui.QWidget ):
                 box.setLayout( box_layout )
                 widget.filter_changed_signal.connect( self._filter_changed )
                 layout.addWidget( box )
-        layout.addStretch()
+        #layout.addStretch()
         self._expanded_search.setLayout( layout )
         self._expanded_filters_created = True
 
     def _filter_changed(self):
+        assert object_thread( self )
         self.filters_changed_signal.emit()
 
     def decorate_query(self, query):
@@ -407,6 +549,7 @@ class HeaderWidget( QtGui.QWidget ):
 
     @QtCore.pyqtSlot()
     def expand_search_options(self):
+        assert object_thread( self )
         if self._expanded_search.isHidden():
             if not self._expanded_filters_created:
                 post( self._admin.get_expanded_search_fields, 
@@ -415,8 +558,8 @@ class HeaderWidget( QtGui.QWidget ):
         else:
             self._expanded_search.hide()
 
-    @gui_function
     def setNumberOfRows( self, rows ):
+        assert object_thread( self )
         if self.number_of_rows:
             self.number_of_rows.setNumberOfRows( rows )
 
@@ -426,18 +569,20 @@ class SplitterHandle( QtGui.QSplitterHandle ):
     
     def __init__ (self, orientation, splitter, widget_to_hide = None):
         super(SplitterHandle, self).__init__ (orientation, splitter)
+        assert object_thread( self )
         self.setToolTip('Click to close')
         self._widget_to_hide = widget_to_hide
         
     def mousePressEvent(self, event):
+        assert object_thread( self )
         splitter = self.splitter()
-        print splitter.count()
         splitter.widget( splitter.count() - 1 ).hide()
         
 class Splitter(QtGui.QSplitter):
     """Custom implementation of QSplitter to use the custom SplitterHandle"""
     
     def createHandle(self):
+        assert object_thread( self )
         return SplitterHandle( self.orientation(), self, self._widget_to_hide )
     
 class TableView( AbstractView  ):
@@ -504,6 +649,7 @@ class TableView( AbstractView  ):
                   search_text = None, 
                   parent = None ):
         super(TableView, self).__init__( parent )
+        assert object_thread( self )
         self.admin = admin
         self.application_gui_context = gui_context
         self.gui_context = gui_context
@@ -557,6 +703,7 @@ class TableView( AbstractView  ):
 
     @QtCore.pyqtSlot()
     def activate_search(self):
+        assert object_thread( self )
         self.header.search.setFocus(QtCore.Qt.ShortcutFocusReason)
 
     @model_function
@@ -564,8 +711,8 @@ class TableView( AbstractView  ):
         return self.title_format % {'verbose_name_plural':self.admin.get_verbose_name_plural()}
 
     @QtCore.pyqtSlot(object)
-    @gui_function
     def setSubclassTree( self, subclasses ):
+        assert object_thread( self )
         if len( subclasses ) > 0:
             from inheritance import SubclassTree
             splitter = self.findChild(QtGui.QWidget, 'splitter' )
@@ -576,6 +723,15 @@ class TableView( AbstractView  ):
     @QtCore.pyqtSlot(int)
     def sectionClicked( self, section ):
         """emits a row_selected signal"""
+        assert object_thread( self )
+        #
+        # close the table editor before opening a form or such
+        #
+        # Qt seems to crash in certain cases when the editor is open and the
+        # underlying model is changed
+        #
+        if self.table:
+            self.table.close_editor()
         self.admin.list_action.gui_run( self.gui_context )
 
     def create_table_model( self, admin ):
@@ -591,10 +747,10 @@ class TableView( AbstractView  ):
         return self.table.model()
 
     @QtCore.pyqtSlot( object )
-    @gui_function
     def set_admin( self, admin ):
         """Switch to a different subclass, where admin is the admin object of the
         subclass"""
+        assert object_thread( self )
         logger.debug('set_admin called')
         self.admin = admin
         if self.table:
@@ -624,11 +780,12 @@ class TableView( AbstractView  ):
 
     @QtCore.pyqtSlot()
     def on_keyboard_selection_signal(self):
+        assert object_thread( self )
         self.sectionClicked( self.table.currentIndex().row() )
 
     @QtCore.pyqtSlot()
-    @gui_function
     def tableLayoutChanged( self ):
+        assert object_thread( self )
         logger.debug('tableLayoutChanged')
         model = self.table.model()
         if self.header:
@@ -641,15 +798,18 @@ class TableView( AbstractView  ):
 
     def closeEvent( self, event ):
         """reimplements close event"""
+        assert object_thread( self )
         logger.debug( 'tableview closed' )
         event.accept()
 
     def selectTableRow( self, row ):
         """selects the specified row"""
+        assert object_thread( self )
         self.table.selectRow( row )
 
     def getColumns( self ):
         """return the columns to be displayed in the table view"""
+        assert object_thread( self )
         return self.admin.get_columns()
 
     def getTitle( self ):
@@ -658,6 +818,7 @@ class TableView( AbstractView  ):
 
     @QtCore.pyqtSlot(object)
     def _set_query(self, query_getter):
+        assert object_thread( self )
         if isinstance(self.table.model(), QueryTableProxy):
             self.table.model().setQuery(query_getter)
         self.table.clearSelection()
@@ -665,6 +826,7 @@ class TableView( AbstractView  ):
     @QtCore.pyqtSlot()
     def refresh(self):
         """Refresh the whole view"""
+        assert object_thread( self )
         post( self.get_admin, self.set_admin )
 
     @QtCore.pyqtSlot()
@@ -690,6 +852,7 @@ class TableView( AbstractView  ):
     @QtCore.pyqtSlot(str)
     def startSearch( self, text ):
         """rebuilds query based on filtering text"""
+        assert object_thread( self )
         from camelot.view.search import create_entity_search_query_decorator
         logger.debug( 'search %s' % text )
         self.search_filter = create_entity_search_query_decorator( self.admin, unicode(text) )
@@ -698,14 +861,15 @@ class TableView( AbstractView  ):
     @QtCore.pyqtSlot()
     def cancelSearch( self ):
         """resets search filtering to default"""
+        assert object_thread( self )
         logger.debug( 'cancel search' )
         self.search_filter = lambda q: q
         self.rebuild_query()
 
     @QtCore.pyqtSlot(object)
-    @gui_function
     def set_filters_and_actions( self, filters_and_actions ):
         """sets filters for the tableview"""
+        assert object_thread( self )
         filters, actions = filters_and_actions
         from camelot.view.controls.filterlist import FilterList
         from camelot.view.controls.actionsbox import ActionsBox
@@ -738,6 +902,8 @@ class TableView( AbstractView  ):
 
     @QtCore.pyqtSlot()
     def focusTable(self):
+        assert object_thread( self )
         if self.table and self.table.model().rowCount() > 0:
             self.table.setFocus()
             self.table.selectRow(0)
+
