@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -46,7 +46,7 @@ from camelot.view.controls.combobox_input_dialog import ComboBoxInputDialog
 from camelot.core.exception import UserException
 from camelot.core.utils import ugettext as _
 from camelot.view.model_thread.signal_slot_model_thread import SignalSlotModelThread
-from camelot.view.model_thread import gui_function
+from camelot.view.model_thread import object_thread
 from camelot.core.dbprofiles import fetch_profiles, use_chosen_profile, \
     store_profiles, get_network_proxy, last_used_profile
 
@@ -158,8 +158,8 @@ allow all languages
         timer.start()
         self.new_network_request()
 
-    @gui_function
     def create_labels_and_widgets(self):
+        assert object_thread( self )
         self.profile_label = QLabel(_('Profile Name:'))
         self.dialect_label = QLabel(_('Driver:'))
         self.host_label = QLabel(_('Server Host:'))
@@ -512,3 +512,4 @@ allow all languages
         if self.profiles:
             store_profiles(self.profiles)
             os.execv(sys.executable, [sys.executable] + sys.argv)
+
