@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -32,7 +32,7 @@ logger = logging.getLogger( 'camelot.view.forms' )
 
 from PyQt4 import QtCore, QtGui
 
-from camelot.view.model_thread import gui_function
+from camelot.core.exception import log_programming_error
 
 class Form( list ):
     """Base Form class to put fields on a form.  The base class of a form is
@@ -40,31 +40,24 @@ a list.  So the form itself is nothing more than a list of field names or
 sub-forms.  A form can thus be manipulated using the list's method such as
 append or insert.
     
-A form can be converted to a
-QT widget by calling its render method.  The base form uses the QFormLayout
-to render a form::
+A form can be converted to a `Qt` widget by calling its `render` method.
 
-    class Admin(EntityAdmin):
-      form_display = Form(['title', 'short_description', 'director', 'release_date'])
+Forms are defined using the `form_display` attribute of an `Admin` class::
+
+    class Admin( EntityAdmin ):
+        form_display = Form( [ 'title', 'short_description', 
+                               'release_date' ] )
+                               
+and takes these parameters :
+
+    :param content: an iterable with field names or sub-forms to render
+    :param columns: the number of columns in which to order the fields.
+
+.. image:: /_static/form/form.png
 
-..image:: /_static/form/form.png
 """
 
     def __init__( self, content, scrollbars = False, columns = 1  ):
-        """
-        :param content: an iterable with the field names and forms to render
-        :param columns: the number of columns in which to order the fields.
-
-        eg : with 2 columns, the fields ['street', 'city', 'country'] will
-        be ordered as :
-
-        +-------------+--------------+
-        | street      | city         |
-        +-------------+--------------+
-        | country     |              |
-        +-------------+--------------+
-
-        """
         super(Form, self).__init__( content )
         self._scrollbars = scrollbars
         self._columns = columns
@@ -88,7 +81,7 @@ to render a form::
         inherited forms.
 
         :param original_field: the name of the field to be removed
-        :return: True if the field was found and removed
+        :return: `True` if the field was found and removed
         """
         for c in self:
             if isinstance( c, Form ):
@@ -104,7 +97,7 @@ to render a form::
 
         :param original_field : the name of the field to be replace
         :param new_field : the name of the new field
-        :return: True if the original field was found and replaced.
+        :return: `True` if the original field was found and replaced
         """
         for i, c in enumerate( self ):
             if isinstance( c, Form ):
@@ -120,55 +113,6 @@ to render a form::
     def __unicode__( self ):
         return 'Form(%s)' % ( u','.join( unicode( c ) for c in self ) )
 
-    def render_ooxml( self, obj, delegates ):
-        """Generator for lines of text in Office Open XML representing this form, using tables
-        :param obj: the object or entity that will be rendered
-        :param delegates: a dictionary mapping field names to their delegate
-        """
-        yield '<w:tbl>'
-        yield '    <w:tblPr>'
-        yield '      <w:tblW w:w="0" w:type="auto"/>'
-        yield '      <w:tblBorders>'
-        yield '          <w:top w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
-        yield '          <w:left w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
-        yield '          <w:bottom w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
-        yield '          <w:right w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
-        yield '          <w:insideH w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
-        yield '          <w:insideV w:val="single" w:sz="4" wx:bdrwidth="10" w:space="0" w:color="auto"/>'
-        yield '        </w:tblBorders>'
-        yield '      <w:tblLook w:val="04A0"/>'
-        yield '    </w:tblPr>'
-        yield '    <w:tblGrid>'
-        yield '      <w:gridCol w:w="4811"/>'
-        yield '      <w:gridCol w:w="4811"/>'
-        yield '    </w:tblGrid>'
-        yield '    <w:tr>'
-        for index, field in enumerate(self):
-            if index % self._columns == 0 and index != 0:
-                yield '    </w:tr><w:tr>'
-            yield '<w:tc>'
-            yield '  <w:tcPr>'
-            yield '    <w:tcW w:w="4811" w:type="dxa"/>'
-            yield '    <w:shd w:val="clear" w:color="auto" w:fill="auto"/>'
-            yield '  </w:tcPr>'
-            yield '<w:p>'
-            lines = []
-            if isinstance(field, Label):
-                lines = field.render_ooxml()
-            elif isinstance(field, Form):
-                lines = field.render_ooxml( obj, delegates )
-            elif isinstance(field, basestring):
-                delegate = delegates[field]
-                value = getattr(obj, field)
-                lines = delegate.render_ooxml(value)
-            for line in lines:
-                yield line
-            yield '</w:p>'
-            yield '</w:tc>'
-        yield '</w:tr>'
-        yield '</w:tbl>'
-
-    @gui_function
     def render( self, widgets, parent = None, toplevel = False):
         """
         :param widgets: a :class:`camelot.view.controls.formview.FormEditors` object
@@ -179,7 +123,7 @@ to render a form::
             while a non toplevel form is only expanding if it contains other
             expanding elements.
 
-        :return: a QWidget into which the form is rendered
+        :return: a :class:`QtGui.QWidget` into which the form is rendered
         """
         logger.debug( 'rendering %s' % (self.__class__.__name__) )
         from camelot.view.controls.editors.wideeditor import WideEditor
@@ -235,6 +179,8 @@ to render a form::
                         if layout_item_widget and layout_item_widget.sizePolicy().verticalPolicy() == QtGui.QSizePolicy.Expanding:
                             has_vertical_expanding_row = True
                     form_layout.addLayout( f, c.row, c.col, row_span, col_span )
+                elif isinstance( f, QtGui.QLayoutItem ):
+                    form_layout.addItem( f )
                 else:
                     form_layout.addWidget( f, c.row, c.col, row_span, col_span )
                     size_policy = f.sizePolicy()
@@ -260,7 +206,7 @@ to render a form::
                         c.next_col()
                     size_policy = editor.sizePolicy()
                 else:
-                    logger.warning('ProgrammingError : widgets should contain a widget for field %s'%unicode(field))
+                    log_programming_error( logger, 'widgets should contain a widget for field %s'%unicode(field) )
             if size_policy and size_policy.verticalPolicy() == QtGui.QSizePolicy.Expanding:
                 has_vertical_expanding_row = True
 
@@ -288,14 +234,14 @@ to render a form::
         
         return form_widget
 
-
 class Label( Form ):
-    """Render a label with a QLabel"""
+    """Render a label using a :class:`QtGui.QLabel`"""
 
     def __init__( self, label, alignment='left', style=None):
         """
         :param label : string to be displayed in the label
-        :param alignment : alignment of text in the label. values that make sense 'left', 'right' or 'center'
+        :param alignment : alignment of text in the label. values that make 
+            sense 'left', 'right' or 'center'
         :param style : string of cascading stylesheet instructions
         """
         super( Label, self ).__init__( [] )
@@ -303,14 +249,6 @@ class Label( Form ):
         self.alignment = alignment
         self.style = style
 
-    def render_ooxml( self ):
-        """Generator for label text in Office Open XML representing this form"""
-        yield '<w:r>'
-        yield '  <w:t>%s</w:t>' % self.label
-        yield '</w:r>'
-
-
-    @gui_function
     def render( self, widgets, parent = None, toplevel = False ):
         from PyQt4 import QtGui
         if self.style:
@@ -388,7 +326,7 @@ the moment the tab is shown.
 
 class TabForm( Form ):
     """
-Render forms within a QTabWidget::
+Render forms within a :class:`QtGui.QTabWidget`::
 
     from = TabForm([('First tab', ['title', 'short_description']),
                     ('Second tab', ['director', 'release_date'])])
@@ -469,7 +407,6 @@ Render forms within a QTabWidget::
             for field in form._get_fields_from_form():
                 yield field
 
-    @gui_function
     def render( self, widgets, parent = None, toplevel = False ):
         logger.debug( 'rendering %s' % self.__class__.__name__ )
         widget = DelayedTabWidget( widgets, self.tabs, parent )
@@ -508,7 +445,6 @@ class HBoxForm( Form ):
             for field in form._get_fields_from_form():
                 yield field
 
-    @gui_function
     def render( self, widgets, parent = None, toplevel = False ):
         logger.debug( 'rendering %s' % self.__class__.__name__ )
         widget = QtGui.QWidget( parent )
@@ -517,6 +453,8 @@ class HBoxForm( Form ):
             f = form.render( widgets, widget, False )
             if isinstance( f, QtGui.QLayout ):
                 form_layout.addLayout( f )
+            elif isinstance( f, QtGui.QLayoutItem ):
+                form_layout.addItem( f )
             else:
                 form_layout.addWidget( f )
         widget.setLayout( form_layout )
@@ -553,7 +491,6 @@ class VBoxForm( Form ):
     def __unicode__( self ):
         return 'VBoxForm [ %s\n         ]' % ( '         \n'.join( [unicode( form ) for form in self.rows] ) )
 
-    @gui_function
     def render( self, widgets, parent = None, toplevel = False ):
         logger.debug( 'rendering %s' % self.__class__.__name__ )
         widget = QtGui.QWidget( parent )
@@ -562,6 +499,8 @@ class VBoxForm( Form ):
             f = form.render( widgets, widget, False )
             if isinstance( f, QtGui.QLayout ):
                 form_layout.addLayout( f )
+            elif isinstance( f, QtGui.QLayoutItem ):
+                form_layout.addItem( f )
             else:
                 form_layout.addWidget( f )
         widget.setLayout( form_layout )
@@ -576,7 +515,7 @@ class ColumnSpan( Form ):
 
 class GridForm( Form ):
     """Put different fields into a grid, without a label.  Row or column labels can be added
-  using the Label form::
+  using the :class:`Label` form::
 
     GridForm([['title', 'short_description'], ['director','release_date']])
 
@@ -610,7 +549,6 @@ class GridForm( Form ):
         for row, additional_field in zip(self._grid, column):
             row.append(additional_field)
 
-    @gui_function
     def render( self, widgets, parent = None, toplevel = False ):
         widget = QtGui.QWidget( parent )
         grid_layout = QtGui.QGridLayout()
@@ -618,16 +556,22 @@ class GridForm( Form ):
             skip = 0
             for j, field in enumerate( row ):
                 num = 1
+                col = j + skip
                 if isinstance( field, ColumnSpan ):
                     num = field.num
                     field = field.field
-
                 if isinstance( field, Form ):
-                    grid_layout.addWidget( field.render( widgets, parent ), i, j + skip, 1, num )
+                    form = field.render( widgets, parent )
+                    if isinstance( form, QtGui.QWidget ):
+                        grid_layout.addWidget( form, i, col, 1, num )
+                    elif isinstance( form, QtGui.QLayoutItem ):
+                        grid_layout.addItem( form, i, col, 1, num )
+                    elif isinstance( form, QtGui.QLayout ):
+                        grid_layout.addLayout( form, i, col, 1, num )
                     skip += num - 1
                 else:
                     editor = widgets.create_editor( field, widget )
-                    grid_layout.addWidget( editor, i, j + skip, 1, num )
+                    grid_layout.addWidget( editor, i, col, 1, num )
                     skip += num - 1
 
         widget.setLayout( grid_layout )
@@ -643,11 +587,21 @@ class WidgetOnlyForm( Form ):
         assert isinstance( field, ( str, unicode ) )
         super( WidgetOnlyForm, self ).__init__( [field] )
 
-    @gui_function
     def render( self, widgets, parent = None, toplevel = False ):
         logger.debug( 'rendering %s' % self.__class__.__name__ )
         editor = widgets.create_editor( self.get_fields()[0], parent )
         return editor
+    
+class Stretch( Form ):
+    """A stretchable space with zero minimum size, this is able to fill a gap
+    in the form if there are no other items to fill this space.
+    """
+    
+    def __init__( self ):
+        super( Stretch, self ).__init__( [] )
+        
+    def render( self, widgets, parent = None, toplevel = False ):
+        return QtGui.QSpacerItem( 0, 0, vPolicy = QtGui.QSizePolicy.Expanding )
 
 class GroupBoxForm( Form ):
     """
@@ -667,7 +621,6 @@ class GroupBoxForm( Form ):
             content = [content]
         Form.__init__( self, content, scrollbars, columns=columns )
 
-    @gui_function
     def render( self, widgets, parent = None, toplevel = False ):
         widget = QtGui.QGroupBox( unicode(self.title), parent )
         layout = QtGui.QVBoxLayout()
@@ -690,5 +643,3 @@ def structure_to_form( structure ):
     if isinstance( structure, Form ):
         return structure
     return Form( structure )
-
-
