@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -28,6 +28,7 @@ logger = logging.getLogger('camelot.view.mainwindow')
 from PyQt4.QtCore import Qt
 from PyQt4 import QtGui, QtCore
 
+from camelot.view.controls.busy_widget import BusyWidget
 from camelot.view.controls.navpane2 import NavigationPane
 from camelot.view.model_thread import post
 
@@ -39,6 +40,10 @@ class MainWindow(QtGui.QMainWindow):
     :param gui_context: an :class:`camelot.admin.action.application_action.ApplicationActionGuiContext`
         object
     :param parent: a :class:`QtGui.QWidget` object or :class:`None` 
+    
+    .. attribute:: splash_screen 
+        a :class:`QtGui.QWidget` that needs to be closed when
+        the main window is shown.
     """
 
     def __init__(self, gui_context, parent=None):
@@ -46,6 +51,7 @@ class MainWindow(QtGui.QMainWindow):
         logger.debug('initializing main window')
         QtGui.QMainWindow.__init__(self, parent)
 
+        self.splash_screen = None
         self.toolbars = []
         self.nav_pane = None
         self.app_admin = gui_context.admin.get_application_admin()
@@ -84,9 +90,6 @@ class MainWindow(QtGui.QMainWindow):
               self.set_bottom_toolbar_actions,
               args = (Qt.BottomToolBarArea,) )
 
-        logger.debug('creating status bar')
-        self.create_status_bar()
-
         logger.debug('reading saved settings')
         self.read_settings()
         
@@ -97,6 +100,25 @@ class MainWindow(QtGui.QMainWindow):
 
         logger.debug('initialization complete')
 
+    @QtCore.pyqtSlot()
+    def show( self ):
+        """This method wait until the main window is completely set up, and
+        only then shows it.  This is a workaround for a bug in Qt on OS X
+        
+        https://bugreports.qt.nokia.com/browse/QTBUG-18567
+        
+        """
+        post( lambda:None, self._delayed_show )
+        
+    @QtCore.pyqtSlot(object)
+    def _delayed_show( self, _o ):
+        """Call to the underlying :meth:`QMainWindow.show`, to be used in
+        :meth:`MainWindow.show`
+        """
+        super( MainWindow, self ).show()
+        if self.splash_screen:
+            self.splash_screen.close()
+        
     @QtCore.pyqtSlot()
     def unmaximize_view( self ):
         """Show the navigation pane and the menu bar if they exist """
@@ -187,6 +209,7 @@ class MainWindow(QtGui.QMainWindow):
                     qaction.triggered.connect( self.action_triggered )
                 toolbar.addAction( qaction )
             self.toolbars.append( toolbar )
+            toolbar.addWidget( BusyWidget() )
                 
     @QtCore.pyqtSlot( object )
     def set_left_toolbar_actions( self, toolbar_actions ):
@@ -213,9 +236,10 @@ class MainWindow(QtGui.QMainWindow):
         model_context = gui_context.create_model_context()
         for toolbar in self.toolbars:
             for qaction in toolbar.actions():
-                post( qaction.action.get_state,
-                      qaction.set_state,
-                      args = ( model_context, ) )
+                if isinstance( qaction, ActionAction ):
+                    post( qaction.action.get_state,
+                          qaction.set_state,
+                          args = ( model_context, ) )
         menu_bar = self.menuBar()
         if menu_bar:
             for qaction in menu_bar.findChildren( ActionAction ):
@@ -248,12 +272,6 @@ class MainWindow(QtGui.QMainWindow):
         else:
             self.navpane = None
 
-    def create_status_bar( self ):
-        from controls.statusbar import StatusBar
-        statusbar = StatusBar( self )
-        self.setStatusBar( statusbar )
-        statusbar.showMessage( _('Ready'), 5000 )
-
     def closeEvent( self, event ):
         from camelot.view.model_thread import get_model_thread
         model_thread = get_model_thread()
@@ -263,3 +281,4 @@ class MainWindow(QtGui.QMainWindow):
         model_thread.stop()
         super( MainWindow, self ).closeEvent( event )
         QtCore.QCoreApplication.exit(0)
+
