@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -24,7 +24,6 @@
 
 from functools import wraps
 
-from PyQt4 import QtGui
 from PyQt4 import QtCore
 
 import logging
@@ -38,7 +37,25 @@ verify_threads = True
 class ModelThreadException(Exception):
     pass
 
-
+def object_thread( self ):
+    """Funtion to verify if a call to an object is made in the thread of this
+    object, to be used in assert statements.  Example ::
+    
+        class FooObject( QtCore.QObject ):
+        
+            def do_something( self ):
+                assert object_thread( self )
+                print 'safe method call'
+    
+    :param self: a :class:`QtCore.QObject` instance.
+    :return True: if the thread of self is the current thread 
+    
+    The approach with assert statements is prefered over decorators,
+    since decorators hide part of the method signature from the sphinx
+    documentation.
+    """
+    return self.thread() == QtCore.QThread.currentThread()
+    
 def model_function(original_function):
     """Decorator to ensure a function is only called from within the model
     thread. If this function is called in another thread, an exception will be
@@ -60,25 +77,6 @@ def model_function(original_function):
 
     return wrapper
 
-def gui_function(original_function):
-    """Decorator to ensure a function is only called from within the gui
-    thread. If this function is called in another thread, an exception will be
-    thrown"""
-
-    def in_gui_thread():
-        """return wether current thread is gui thread"""
-        gui_thread = QtGui.QApplication.instance().thread()
-        current_thread = QtCore.QThread.currentThread()
-        return gui_thread == current_thread
-
-    @wraps(original_function)
-    def wrapper(*args, **kwargs):
-        assert (not verify_threads) or in_gui_thread()
-        return original_function(*args, **kwargs)
-
-    return wrapper
-
-
 def setup_model():
     """Call the setup_model function in the settings"""
     from camelot.core.conf import settings
@@ -173,3 +171,4 @@ def post(request, response=None, exception=None, args=()):
     """Post a request and a response to the default model thread"""
     mt = get_model_thread()
     mt.post(request, response, exception, args)
+
