@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -38,14 +38,14 @@ import itertools
 from PyQt4.QtCore import Qt, QThread
 from PyQt4 import QtGui, QtCore
 
-from camelot.core.utils import is_deleted
-from camelot.core.files.storage import StoredFile
+from camelot.core.exception import log_programming_error
+from camelot.core.utils import is_deleted, variant_to_pyobject
 from camelot.view.art import Icon
 from camelot.view.fifo import Fifo
 from camelot.view.controls import delegates
 from camelot.view.controls.exception import register_exception
 from camelot.view.remote_signals import get_signal_handler
-from camelot.view.model_thread import gui_function, \
+from camelot.view.model_thread import object_thread, \
                                       model_function, post
 
 from camelot.core.files.storage import StoredImage
@@ -69,7 +69,6 @@ class DelayedProxy( object ):
         self._collection_getter = collection_getter
         self._columns_getter = columns_getter
 
-    @gui_function
     def __call__( self ):
         return CollectionProxy( self._admin,
                                 self._collection_getter,
@@ -110,8 +109,10 @@ def strip_data_from_object( obj, columns ):
             else:
                 field_value = getter( obj )
         except (Exception, RuntimeError, TypeError, NameError), e:
-            logger.error('ProgrammingError : could not get field %s of object of type %s'%(col[0], obj.__class__.__name__),
-                         exc_info=e)
+            message = "could not get field '%s' of object of type %s"%(col[0], obj.__class__.__name__)
+            log_programming_error( logger, 
+                                   message,
+                                   exc_info = e )
         finally:
             row_data.append( field_value )
     return row_data
@@ -125,30 +126,37 @@ def stripped_data_to_unicode( stripped_data, obj, static_field_attributes, dynam
 
     for field_data, static_attributes, dynamic_attributes in zip( stripped_data, static_field_attributes, dynamic_field_attributes ):
         unicode_data = u''
-        choices = dynamic_attributes.get( 'choices', static_attributes.get('choices', None))
-        if 'unicode_format' in static_attributes:
-            unicode_format = static_attributes['unicode_format']
-            if field_data != None:
-                unicode_data = unicode_format( field_data )
-        elif choices:
-            unicode_data = field_data
-            for key, value in choices:
-                if key == field_data:
-                    unicode_data = value
-        elif isinstance( field_data, list ):
-            unicode_data = u'.'.join( [unicode( e ) for e in field_data] )
-        elif isinstance( field_data, datetime.datetime ):
-            # datetime should come before date since datetime is a subtype of date
-            if field_data.year >= 1900:
-                unicode_data = field_data.strftime( '%d/%m/%Y %H:%M' )
-        elif isinstance( field_data, datetime.date ):
-            if field_data.year >= 1900:
-                unicode_data = field_data.strftime( '%d/%m/%Y' )
-        elif isinstance( field_data, StoredImage):
-            unicode_data = field_data.checkout_thumbnail(100, 100)
-        elif field_data != None:
-            unicode_data = unicode( field_data )
-        row_data.append( unicode_data )
+        try:
+            choices = dynamic_attributes.get( 'choices', static_attributes.get('choices', None))
+            if 'unicode_format' in static_attributes:
+                unicode_format = static_attributes['unicode_format']
+                if field_data != None:
+                    unicode_data = unicode_format( field_data )
+            elif choices:
+                unicode_data = field_data
+                for key, value in choices:
+                    if key == field_data:
+                        unicode_data = value
+            elif isinstance( field_data, list ):
+                unicode_data = u'.'.join( [unicode( e ) for e in field_data] )
+            elif isinstance( field_data, datetime.datetime ):
+                # datetime should come before date since datetime is a subtype of date
+                if field_data.year >= 1900:
+                    unicode_data = field_data.strftime( '%d/%m/%Y %H:%M' )
+            elif isinstance( field_data, datetime.date ):
+                if field_data.year >= 1900:
+                    unicode_data = field_data.strftime( '%d/%m/%Y' )
+            elif isinstance( field_data, StoredImage):
+                unicode_data = field_data.checkout_thumbnail(100, 100)
+            elif field_data != None:
+                unicode_data = unicode( field_data )
+        except (Exception, RuntimeError, TypeError, NameError), e:
+            log_programming_error( logger,
+                                   "Could not get view data for field '%s' with of object of type %s"%( static_attributes['name'],
+                                                                                                        obj.__class__.__name__),
+                                   exc_info = e )
+        finally:
+            row_data.append( unicode_data )
 
     return row_data
 
@@ -170,7 +178,7 @@ class SortingRowMapper( dict ):
         except KeyError:
             return row
 
-class CollectionProxy( QtCore.QAbstractTableModel ):
+class CollectionProxy( QtGui.QProxyModel ):
     """The CollectionProxy contains a limited copy of the data in the actual
     collection, usable for fast visualisation in a QTableView
 
@@ -192,7 +200,6 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
     exception_signal = QtCore.pyqtSignal(object)
     rows_removed_signal = QtCore.pyqtSignal()
 
-    @gui_function
     def __init__( self, 
                   admin, 
                   collection_getter, 
@@ -222,12 +229,22 @@ query of the tableview, and the first of the form, the object might have changed
 position in the query.
 """
         super(CollectionProxy, self).__init__()
+        assert object_thread( self )
         from camelot.view.model_thread import get_model_thread
+        #
+        # The source model will contain the actual data stripped from the
+        # objects in the collection.
+        #
+        self.source_model = QtGui.QStandardItemModel()
+        self.setModel( self.source_model )
+        
         self.logger = logging.getLogger(logger.name + '.%s'%id(self))
         self.logger.debug('initialize query table for %s' % (admin.get_verbose_name()))
         self._mutex = QtCore.QMutex()
         self.admin = admin
+        self.settings = self.admin.get_settings()
         self._horizontal_header_height = QtGui.QFontMetrics( self._header_font_required ).height() + 10
+        self._header_font_metrics = QtGui.QFontMetrics( self._header_font )
         vertical_header_font_height = QtGui.QFontMetrics( self._header_font ).height()
         self._vertical_header_height = vertical_header_font_height * self.admin.lines_per_row + 10
         self.iconSize = QtCore.QSize( vertical_header_font_height,
@@ -238,7 +255,6 @@ position in the query.
             self.form_icon = QtCore.QVariant()
         self.validator = admin.create_validator( self )
         self._collection_getter = collection_getter
-        self.column_count = 0
         self.flush_changes = flush_changes
         self.delegate_manager = None
         self.mt = get_model_thread()
@@ -283,6 +299,61 @@ position in the query.
             post( self.getRowCount, self.setRowCount )
         self.logger.debug( 'initialization finished' )
 
+    #
+    # Reimplementation of methods of QProxyModel, because for now, we only
+    # use the proxy to store header data
+    # 
+    # Subsequent calls to this function should return the same index
+    #
+    def index( self, row, col, parent = QtCore.QModelIndex() ):
+        assert object_thread( self )
+        if self.hasIndex( row, col, parent):
+            #
+            # indexes are considered equal when their row, column and internal
+            # pointer are equal.  therefor set the internal pointer always to 0.
+            #
+            return self.createIndex( row, col, 0 )
+        return QtCore.QModelIndex()
+    
+    def parent( self, child ):
+        assert object_thread( self )
+        return QtCore.QModelIndex()
+    
+    def rowCount( self, index = None ):
+        return self._rows
+    
+    def hasChildren( self, parent ):
+        assert object_thread( self )
+        return False
+    
+    #
+    # end or reimplementation
+    #
+    
+    #
+    # begin functions related to drag and drop
+    #
+    
+    def mimeTypes( self ):
+        assert object_thread( self )
+        if self.admin.drop_action != None:
+            return self.admin.drop_action.drop_mime_types
+        
+    def supportedDropActions( self ):
+        assert object_thread( self )
+        if self.admin.drop_action != None:
+            return Qt.CopyAction | Qt.MoveAction | Qt.LinkAction
+        return None
+    
+    def dropMimeData( self, mime_data, action, row, column, parent ):
+        assert object_thread( self )
+        #print mime_data, [unicode(f) for f in mime_data.formats()]
+        return True
+    
+    #
+    # end of drag and drop related functions
+    #
+    
     @property
     def max_number_of_rows(self):
         """The maximum number of rows to be displayed at once"""
@@ -319,19 +390,21 @@ position in the query.
         rows = len( set( self.get_collection() ) )
         return rows
 
-    @gui_function
     def refresh( self ):
+        assert object_thread( self )
         post( self.getRowCount, self._refresh_content )
 
     @QtCore.pyqtSlot(int)
-    @gui_function
     def _refresh_content(self, rows ):
+        assert object_thread( self )
+        locker = QtCore.QMutexLocker(self._mutex)
         self.display_cache = Fifo( 10 * self.max_number_of_rows )
         self.edit_cache = Fifo( 10 * self.max_number_of_rows )
         self.attributes_cache = Fifo( 10 * self.max_number_of_rows )
-        locker = QtCore.QMutexLocker(self._mutex)
         self.rows_under_request = set()
         self.unflushed_rows = set()
+        # once the cache has been cleared, no updates ought to be accepted
+        self._update_requests = list()
         locker.unlock()
         self.setRowCount( rows )
 
@@ -344,24 +417,25 @@ position in the query.
     def get_collection( self ):
         return self._collection_getter()
 
-    @gui_function
     def handleRowUpdate( self, row ):
         """Handles the update of a row when this row might be out of date"""
+        assert object_thread( self )
         self.display_cache.delete_by_row( row )
         self.edit_cache.delete_by_row( row )
         self.attributes_cache.delete_by_row( row )
         self.dataChanged.emit( self.index( row, 0 ),
-                               self.index( row, self.column_count ) )
+                               self.index( row, self.columnCount() - 1 ) )
 
     @QtCore.pyqtSlot( object, object )
     def handle_entity_update( self, sender, entity ):
         """Handles the entity signal, indicating that the model is out of
         date"""
+        assert object_thread( self )
         self.logger.debug( '%s %s received entity update signal' % \
                      ( self.__class__.__name__, self.admin.get_verbose_name() ) )
         if sender != self:
             try:
-                row = self.display_cache.get_row_by_entity(entity)
+                row = self.display_cache.get_row_by_entity( entity )
             except KeyError:
                 self.logger.debug( 'entity not in cache' )
                 return
@@ -373,7 +447,7 @@ position in the query.
             def create_entity_update(row, entity):
 
                 def entity_update():
-                    columns = self.getColumns()
+                    columns = self._columns
                     self._add_data(columns, row, entity)
                     return row
 
@@ -387,6 +461,7 @@ position in the query.
     def handle_entity_delete( self, sender, entity ):
         """Handles the entity signal, indicating that the model is out of
         date"""
+        assert object_thread( self )
         self.logger.debug( 'received entity delete signal' )
         if sender != self:
             try:
@@ -400,6 +475,7 @@ position in the query.
     def handle_entity_create( self, sender, entity ):
         """Handles the entity signal, indicating that the model is out of
         date"""
+        assert object_thread( self )
         self.logger.debug( 'received entity create signal' )
         # @todo : decide what to do when a new entity has been created,
         #         probably do nothing
@@ -410,12 +486,14 @@ position in the query.
         """Callback method to set the number of rows
         @param rows the new number of rows
         """
+        assert object_thread( self )
         self._rows = rows
         self.layoutChanged.emit()
 
     def getItemDelegate( self ):
         """:return: a DelegateManager for this model, or None if no DelegateManager yet available
         a DelegateManager will be available once the item_delegate_changed signal has been emitted"""
+        assert object_thread( self )
         self.logger.debug( 'getItemDelegate' )
         return self.delegate_manager
 
@@ -424,100 +502,104 @@ position in the query.
         return self._columns
 
     @QtCore.pyqtSlot(object)
-    @gui_function
     def setColumns( self, columns ):
         """Callback method to set the columns
 
         :param columns: a list with fields to be displayed of the form [('field_name', field_attributes), ...] as
         returned by the getColumns method of the ElixirAdmin class
         """
+        assert object_thread( self )
         self.logger.debug( 'setColumns' )
-        self.column_count = len( columns )
         self._columns = columns
 
         delegate_manager = delegates.DelegateManager()
         delegate_manager.set_columns_desc( columns )
 
         # set a delegate for the vertical header
-        delegate_manager.insertColumnDelegate( -1, delegates.PlainTextDelegate(parent = delegate_manager) )
+        delegate_manager.insertColumnDelegate( -1, delegates.PlainTextDelegate( parent = delegate_manager ) )
+        index = QtCore.QModelIndex()
+        option = QtGui.QStyleOptionViewItem()
+        self.settings.beginGroup( 'column_width' )
+        self.settings.beginGroup( '0' )
 
         #
         # this loop can take a while to complete, so processEvents is called regulary
         #
         for i, c in enumerate( columns ):
-#            if i%10==0:
-#                QtCore.QCoreApplication.processEvents(QtCore.QEventLoop.ExcludeSocketNotifiers, 100)
+            #
+            # Construct the delegate
+            #
             field_name = c[0]
             self.logger.debug( 'creating delegate for %s' % field_name )
-            if 'delegate' in c[1]:
-                try:
-                    delegate = c[1]['delegate']( parent = delegate_manager, **c[1] )
-                except Exception, e:
-                    logger.error('ProgrammingError : could not create delegate for field %s'%field_name, exc_info=e)
-                    delegate = delegates.PlainTextDelegate( parent = delegate_manager, **c[1] )
-                delegate_manager.insertColumnDelegate( i, delegate )
-                continue
-            elif c[1]['python_type'] == str:
-                if c[1]['length']:
-                    delegate = delegates.PlainTextDelegate( parent = delegate_manager, maxlength = c[1]['length'] )
-                    delegate_manager.insertColumnDelegate( i, delegate )
-                else:
-                    delegate = delegates.TextEditDelegate( parent = delegate_manager, **c[1] )
-                    delegate_manager.insertColumnDelegate( i, delegate )
+            try:
+                delegate = c[1]['delegate']( parent = delegate_manager, **c[1] )
+            except Exception, e:
+                log_programming_error( logger, 
+                                       'Could not create delegate for field %s'%field_name,
+                                       exc_info = e )
+                delegate = delegates.PlainTextDelegate( parent = delegate_manager, **c[1] )
+            delegate_manager.insertColumnDelegate( i, delegate )
+            #
+            # Set the header data
+            #
+            header_item = QtGui.QStandardItem()
+            header_item.setData( QtCore.QVariant( unicode(c[1]['name']) ),
+                                 Qt.DisplayRole )
+            if c[1].get( 'nullable', True ) == False:
+                header_item.setData( self._header_font_required,
+                                     Qt.FontRole )
             else:
-                delegate = delegates.PlainTextDelegate(parent = delegate_manager)
-                delegate_manager.insertColumnDelegate( i, delegate )
+                header_item.setData( self._header_font,
+                                     Qt.FontRole )
 
+            settings_width = int( variant_to_pyobject( self.settings.value( field_name, 0 ) ) )
+            label_size = QtGui.QFontMetrics( self._header_font_required ).size( Qt.TextSingleLine, unicode(c[1]['name']) + u' ' )
+            minimal_widths = [ label_size.width() + 10 ]
+            if 'minimal_column_width' in c[1]:
+                minimal_widths.append( self._header_font_metrics.averageCharWidth() * c[1]['minimal_column_width'] )
+            if c[1].get('editable', True) != False:
+                minimal_widths.append( delegate.sizeHint( option, index ).width() )
+            column_width = c[1].get( 'column_width', None )
+            if column_width != None:
+                minimal_widths = [ self._header_font_metrics.averageCharWidth() * column_width ]
+                    
+            if settings_width:
+                header_item.setData( QtCore.QVariant( QtCore.QSize( settings_width, self._horizontal_header_height ) ),
+                                     Qt.SizeHintRole )
+            else:
+                header_item.setData( QtCore.QVariant( QtCore.QSize( max( minimal_widths ), self._horizontal_header_height ) ),
+                                     Qt.SizeHintRole )
+             
+            self.source_model.setHorizontalHeaderItem( i, header_item )
+        
+        self.settings.endGroup()
+        self.settings.endGroup()
         # Only set the delegate manager when it is fully set up
         self.delegate_manager = delegate_manager
         self.item_delegate_changed_signal.emit()
-        self.layoutChanged.emit()
-
-    def rowCount( self, index = None ):
-        return self._rows
-
-    def columnCount( self, index = None ):
-        return self.column_count
-
-    @gui_function
+            
+    def setHeaderData( self, section, orientation, value, role ):
+        assert object_thread( self )
+        if orientation == Qt.Horizontal:
+            if role == Qt.SizeHintRole:
+                width = value.width()
+                self.settings.beginGroup( 'column_width' )
+                self.settings.beginGroup( '0' )
+                self.settings.setValue( self._columns[section][1]['field_name'], 
+                                        width )
+                self.settings.endGroup()
+                self.settings.endGroup()
+        return super( CollectionProxy, self ).setHeaderData( section,
+                                                             orientation,
+                                                             value,
+                                                             role )
+    
     def headerData( self, section, orientation, role ):
         """In case the columns have not been set yet, don't even try to get
         information out of them
         """
-        if orientation == Qt.Horizontal:
-            if section >= self.column_count:
-                return QtCore.QAbstractTableModel.headerData( self, section, orientation, role )
-            c = self.getColumns()[section]
-
-            if role == Qt.DisplayRole:
-                return QtCore.QVariant( unicode(c[1]['name']) )
-
-            elif role == Qt.FontRole:
-                if ( 'nullable' in c[1] ) and \
-                   ( c[1]['nullable'] == False ):
-                    return QtCore.QVariant( self._header_font_required )
-                else:
-                    return QtCore.QVariant( self._header_font )
-
-            elif role == Qt.SizeHintRole:
-                option = QtGui.QStyleOptionViewItem()
-                if self.delegate_manager:
-                    editor_size = self.delegate_manager.sizeHint( option, self.index( 0, section ) )
-                else:
-                    editor_size = QtCore.QSize(0, 0)
-                if 'minimal_column_width' in c[1]:
-                    minimal_column_width = QtGui.QFontMetrics( self._header_font ).size( Qt.TextSingleLine, 'A' ).width()*c[1]['minimal_column_width']
-                else:
-                    minimal_column_width = 100
-                editable = True
-                if 'editable' in c[1]:
-                    editable = c[1]['editable']
-                label_size = QtGui.QFontMetrics( self._header_font_required ).size( Qt.TextSingleLine, unicode(c[1]['name']) + ' ' )
-                size = max( minimal_column_width, label_size.width() + 10 )
-                if editable:
-                    size = max( size, editor_size.width() )
-                return QtCore.QVariant( QtCore.QSize( size, self._horizontal_header_height ) )
-        else:
+        assert object_thread( self )
+        if orientation == Qt.Vertical:
             if role == Qt.SizeHintRole:
                 if self.header_icon != None:
                     return QtCore.QVariant( QtCore.QSize( self.iconSize.width() + 10,
@@ -530,18 +612,38 @@ position in the query.
             elif role == Qt.DisplayRole:
                 if self.header_icon != None:
                     return QtCore.QVariant( '' )
-        return QtCore.QAbstractTableModel.headerData( self, section, orientation, role )
+        return super( CollectionProxy, self ).headerData( section, orientation, role )
 
-    @gui_function
     def sort( self, column, order ):
-        """reimplementation of the QAbstractItemModel its sort function"""
+        """reimplementation of the :class:`QtGui.QAbstractItemModel` its sort function"""
+        assert object_thread( self )
 
         def create_sort(column, order):
 
             def sort():
                 unsorted_collection = [(i,o) for i,o in enumerate(self.get_collection())]
-                key = lambda item:getattr(item[1], self._columns[column][0])
-                unsorted_collection.sort(key=key, reverse=order)
+                field_name = self._columns[column][0]
+                
+                # handle the case of one of the values being None
+                def compare_none( line_1, line_2 ):
+                    key_1, key_2 = None, None
+                    try:
+                        key_1 = getattr( line_1[1], field_name )
+                    except Exception, e:
+                        logger.error( 'could not get attribute %s from object'%field_name, exc_info = e )
+                    try:
+                        key_2 = getattr( line_2[1], field_name )
+                    except Exception, e:
+                        logger.error( 'could not get attribute %s from object'%field_name, exc_info = e )
+                    if key_1 == None and key_2 == None:
+                        return 0
+                    if key_1 == None:
+                        return -1
+                    if key_2 == None:
+                        return 1
+                    return cmp( key_1, key_2 )
+                    
+                unsorted_collection.sort( cmp = compare_none, reverse = order )
                 for j,(i,_o) in enumerate(unsorted_collection):
                     self._sort_and_filter[j] = i
                 return len(unsorted_collection)
@@ -550,7 +652,6 @@ position in the query.
 
         post(create_sort(column, order), self._refresh_content)
 
-    @gui_function
     def data( self, index, role = Qt.DisplayRole):
         """:return: the data at index for the specified role
         This function will return ValueLoading when the data has not
@@ -565,9 +666,10 @@ position in the query.
         is displayed in that specific cell
         
         """
+        assert object_thread( self )
         if not index.isValid() or \
            not ( 0 <= index.row() <= self.rowCount( index ) ) or \
-           not ( 0 <= index.column() <= self.columnCount( index ) ):
+           not ( 0 <= index.column() <= self.columnCount() ):
             return QtCore.QVariant()
         if role in (Qt.EditRole, Qt.DisplayRole):
             if role == Qt.EditRole:
@@ -632,7 +734,7 @@ position in the query.
         #
         return_list = []
         for flushed, row, column, value in update_requests:
-            attribute, field_attributes = self.getColumns()[column]
+            attribute, field_attributes = self._columns[column]
 
             from sqlalchemy.exc import DatabaseError
             new_value = value()
@@ -680,7 +782,6 @@ position in the query.
                 changed = True
             if changed:
                 # update the model
-                model_updated = False
                 try:
                     setattr( o, attribute, new_value )
                     #
@@ -688,7 +789,6 @@ position in the query.
                     # that was not returned before
                     #
                     self.admin.set_defaults( o, include_nullable_fields=False )
-                    model_updated = True
                 except AttributeError, e:
                     self.logger.error( u"Can't set attribute %s to %s" % ( attribute, unicode( new_value ) ), exc_info = e )
                 except TypeError:
@@ -707,37 +807,13 @@ position in the query.
                     except KeyError:
                         pass
                     locker.unlock()
-                    #
-                    # we can only track history if the model was updated, and it was
-                    # flushed before, otherwise it has no primary key yet
-                    #
-                    if model_updated and hasattr(o, 'id') and o.id:
-                        #
-                        # in case of files or relations, we cannot pickle them
-                        #
-                        if isinstance( old_value, StoredFile ):
-                            old_value = old_value.name
-                        if not direction:
-                            from camelot.model.memento import BeforeUpdate
-                            # only register the update when the camelot model is active
-                            if hasattr(BeforeUpdate, 'query'):
-                                from camelot.model.authentication import getCurrentAuthentication
-                                history = BeforeUpdate( model = unicode( self.admin.entity.__name__ ),
-                                                       primary_key = o.id,
-                                                       previous_attributes = {attribute:old_value},
-                                                       authentication = getCurrentAuthentication() )
-
-                                try:
-                                    history.flush()
-                                except DatabaseError, e:
-                                    self.logger.error( 'Programming Error, could not flush history', exc_info = e )
                 # update the cache
-                self._add_data(self.getColumns(), row, o)
+                self._add_data(self._columns, row, o)
                 #@todo: update should only be sent remotely when flush was done
                 self.rsh.sendEntityUpdate( self, o )
                 for depending_obj in self.admin.get_depending_objects( o ):
                     self.rsh.sendEntityUpdate( self, depending_obj )
-                return_list.append(( ( row, 0 ), ( row, len( self.getColumns() ) ) ))
+                return_list.append(( ( row, 0 ), ( row, len( self._columns ) ) ))
             elif flushed:
                 locker.relock()
                 self.logger.debug( 'old value equals new value, no need to flush this object' )
@@ -754,10 +830,11 @@ position in the query.
 
         This function will then be called in the model_thread
         """
+        assert object_thread( self )
         #
         # prevent data of being set in rows not actually in this model
         #
-        if not index.isValid():
+        if (not index.isValid()) or (index.model()!=self):
             return False
         
         if role == Qt.EditRole:
@@ -778,17 +855,22 @@ position in the query.
         return True
 
     @QtCore.pyqtSlot(int)
-    @gui_function
     def _emit_changes( self, row ):
+        assert object_thread( self )
         if row!=None:
-            self.dataChanged.emit( self.index( row, 0 ),
-                                   self.index( row, self.column_count ) )
+            column_count = self.columnCount()
+            top_left = self.index( row, 0 )
+            bottom_right = self.index( row, column_count - 1 )
+            self.dataChanged.emit( top_left, bottom_right )
 
     def flags( self, index ):
         """Returns the item flags for the given index"""
+        assert object_thread( self )
         flags = Qt.ItemIsEnabled | Qt.ItemIsSelectable
         if self._get_field_attribute_value(index, 'editable'):
             flags = flags | Qt.ItemIsEditable
+        if self.admin.drop_action != None:
+            flags = flags | Qt.ItemIsDropEnabled
         return flags
 
     def _add_data(self, columns, row, obj):
@@ -882,7 +964,7 @@ position in the query.
         """Extend the cache around the rows under request"""
         offset, limit = self._offset_and_limit_rows_to_get()
         if limit:
-            columns = self.getColumns()
+            columns = self._columns
             collection = self.get_collection()
             skipped_rows = 0
             for i in range(offset, min(offset + limit + 1, self._rows)):
@@ -1006,7 +1088,6 @@ position in the query.
             self.rsh.sendEntityUpdate( self, depending_obj )
         post( self.getRowCount, self._refresh_content )
 
-    @gui_function
     def remove_rows( self, rows, delete = True ):
         """Remove the entity associated with this row from this collection
         @param rows: a list with the numbers of the rows to remove
@@ -1015,6 +1096,7 @@ position in the query.
         The rows_removed signal will be emitted when the removal was 
         successful, otherwise the exception_signal will be emitted.
         """
+        assert object_thread( self )
         self.logger.debug( 'remove rows' )
 
         def create_delete_function( rows ):
@@ -1037,12 +1119,12 @@ position in the query.
         post( create_delete_function(rows) )
         return True
 
-    @gui_function
     def copy_row( self, row ):
         """Copy the entity associated with this row to the end of the collection
         :param row: the row number
         """
-
+        assert object_thread( self )
+        
         def create_copy_function( row ):
 
             def copy_function():
@@ -1063,7 +1145,7 @@ position in the query.
         :param obj: the object to be added to the collection
         :return: the new number of rows in the collection
         """
-        rows = self.rowCount()
+        rows = self._rows
         row = max( rows - 1, 0 )
         self.beginInsertRows( QtCore.QModelIndex(), row, row )
         self.append( obj )
@@ -1078,31 +1160,22 @@ position in the query.
                 pass
         for depending_obj in self.admin.get_depending_objects( obj ):
             self.rsh.sendEntityUpdate( self, depending_obj )
-# TODO : it's not because an object is added to this list, that it was created
-# it might as well exist already, eg. manytomany relation
-#      from camelot.model.memento import Create
-#      from camelot.model.authentication import getCurrentAuthentication
-#      history = Create(model=unicode(self.admin.entity.__name__),
-#                       primary_key=o.id,
-#                       authentication = getCurrentAuthentication())
-#      elixir.session.flush([history])
-#      self.rsh.sendEntityCreate(self, o)
         self._rows = rows + 1
         #
         # update the cache, so the object can be retrieved
         #
-        columns = self.getColumns()
+        columns = self._columns
         self._add_data( columns, rows, obj )
         self.endInsertRows()
         return self._rows
 
     @QtCore.pyqtSlot(object)
-    @gui_function
     def append_row( self, object_getter ):
         """
         :param object_getter: a function that returns the object to be put in the
         appended row.
         """
+        assert object_thread( self )
 
         def create_append_function( getter ):
 
@@ -1117,9 +1190,8 @@ position in the query.
     def getData( self ):
         """Generator for all the data queried by this proxy"""
         for _i, o in enumerate( self.get_collection() ):
-            yield strip_data_from_object( o, self.getColumns() )
+            yield strip_data_from_object( o, self._columns )
 
     def get_admin( self ):
         """Get the admin object associated with this model"""
         return self.admin
-
