@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -22,6 +22,9 @@
 #
 #  ============================================================================
 
+import os
+import logging
+
 from camelot.view.model_thread import post
 from camelot.core.utils import ugettext as _
 from camelot.view.controls.exception import model_thread_exception_message_box
@@ -29,6 +32,8 @@ from camelot.view.controls.exception import model_thread_exception_message_box
 from PyQt4 import QtGui
 from PyQt4 import QtCore
 
+LOGGER = logging.getLogger('camelot.view.storage')
+
 class OpenFileProgressDialog(QtGui.QProgressDialog):
 
     def __init__(self):
@@ -39,7 +44,6 @@ class OpenFileProgressDialog(QtGui.QProgressDialog):
         self.setRange(0, 0)
 
     def open_path(self, path):
-        import os
         if not os.path.exists(path):
             QtGui.QMessageBox.critical(
                 self,
@@ -55,7 +59,7 @@ class OpenFileProgressDialog(QtGui.QProgressDialog):
             url = QtCore.QUrl(path, QtCore.QUrl.TolerantMode)
         QtGui.QDesktopServices.openUrl(url)
         self.close()
-        
+
     def file_stored(self):
         """Called when the file has been stored at path"""
         self.close()
@@ -89,42 +93,69 @@ def open_stored_file(parent, stored_file):
 
 def save_stored_file(parent, stored_file):
     """Save a stored file as another file"""
-    import os
-    
     settings = QtCore.QSettings()
     default_dir = settings.value('datasource').toString()
     proposal = os.path.join(unicode(default_dir), unicode(stored_file.verbose_name) )
-    
+
     path = QtGui.QFileDialog.getSaveFileName(
         parent, _('Save as'), proposal
     )
 
     if path:
         progress = OpenFileProgressDialog()
-        
+
         def save_as():
             destination = open(path, 'wb')
             destination.write( stored_file.storage.checkout_stream(stored_file).read() )
-     
+
         post(save_as, progress.file_stored, model_thread_exception_message_box)
         progress.exec_()
-       
-def create_stored_file(parent, storage, on_finish, filter='All files (*)'):
+
+def create_stored_file(parent, storage, on_finish, filter='All files (*)',
+                       remove_original = False,
+                       filename = None):
     """Popup a QFileDialog, put the selected file in the storage and
-    return the call on_finish with the StoredFile when done"""
+    return the call on_finish with the StoredFile when done
+    
+    :param on_finish: function that will be called in the gui thread when
+    the file is stored.  the first argument of the function should be the
+    StoredFile
+    
+    :param filename: if None, a dialog will pop up, asking the user for
+    the file, otherwise a string with the name of the file to be stored
+    """
     settings = QtCore.QSettings()
     dir = settings.value('lastpath').toString()
     # use last path saved in settings, if none current dir is used by Qt
-    filename = QtGui.QFileDialog.getOpenFileName(
-        parent, 'Open file', dir, filter
-    )
+    if filename == None:
+        filename = QtGui.QFileDialog.getOpenFileName(
+            parent, 'Open file', dir, filter
+        )
     if filename:
+        filename = unicode( filename )
+        remove = False
+        if remove_original:
+            reply = QtGui.QMessageBox(
+                QtGui.QMessageBox.Warning,
+                _('The file will be stored.'),
+                _('Do you want to remove the original file?'),
+                QtGui.QMessageBox.No | QtGui.QMessageBox.Yes,
+            )
+            reply.setDefaultButton(QtGui.QMessageBox.Yes)
+            reply.exec_()
+            if reply == QtGui.QMessageBox.Yes:
+                remove = True
         # save it back
-        settings.setValue('lastpath', QtCore.QVariant(filename))
+        settings.setValue('lastpath', QtCore.QVariant( os.path.dirname( filename ) ) )
         progress = SaveFileProgressDialog()
 
         def checkin():
-            new_path = storage.checkin(unicode(filename))
+            new_path = storage.checkin( filename )
+            if remove:
+                try:
+                    os.remove( filename )
+                except Exception, e:
+                    LOGGER.warn('could not remove file', exc_info=e)
             return lambda:on_finish(new_path)
 
         post(checkin, progress.finish, model_thread_exception_message_box)
