@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2012 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -33,7 +33,7 @@ logger = logging.getLogger('camelot.view.workspace')
 
 from camelot.admin.action import ApplicationActionGuiContext
 from camelot.core.utils import ugettext as _
-from camelot.view.model_thread import gui_function, post
+from camelot.view.model_thread import object_thread, post
 from camelot.view.controls.action_widget import ( ActionLabel, 
                                                   HOVER_ANIMATION_DISTANCE,
                                                   NOTIFICATION_ANIMATION_DISTANCE )
@@ -251,12 +251,11 @@ class DesktopWorkspace(QtGui.QWidget):
     change_view_mode_signal = QtCore.pyqtSignal()
     last_view_closed_signal = QtCore.pyqtSignal()
 
-    @gui_function
     def __init__(self, app_admin, parent):
         super(DesktopWorkspace, self).__init__(parent)
         self.gui_context = ApplicationActionGuiContext()
         self.gui_context.admin = app_admin
-        self.gui_context.desktop = self
+        self.gui_context.workspace = self
         self._app_admin = app_admin
         
         layout = QtGui.QHBoxLayout()
@@ -386,11 +385,11 @@ class DesktopWorkspace(QtGui.QWidget):
                 index = self._tab_widget.insertTab(index, view, title)
             self._tab_widget.setCurrentIndex(index)
 
-    @gui_function
     def add_view(self, view, icon = None, title = '...'):
         """
         Add a Widget implementing AbstractView to the workspace.
         """
+        assert object_thread( self )
         view.title_changed_signal.connect(self.change_title)
         view.icon_changed_signal.connect(self.change_icon)
         if icon:
@@ -462,3 +461,4 @@ def show_top_level(view, parent):
 
     #view.setWindowModality(QtCore.Qt.WindowModal)
     view.show()
+
