@@ -12,8 +12,8 @@ class MyApplicationAdmin(ApplicationAdmin):
     def get_sections(self):
         
         from camelot.model.memento import Memento
-        from camelot.model.authentication import ( Person, Organization, 
-                                                   PartyCategory )
+        from camelot.model.party import ( Person, Organization, 
+                                          PartyCategory )
         from camelot.model.i18n import Translation
         
         from camelot_example.model import Movie, Tag, VisitorReport
@@ -69,7 +69,7 @@ class MiniApplicationAdmin( MyApplicationAdmin ):
 
     def get_toolbar_actions( self, toolbar_area ):
         from PyQt4.QtCore import Qt
-        from camelot.model.authentication import Person
+        from camelot.model.party import Person
         from camelot.admin.action import application_action, list_action
         from model import Movie
         
@@ -99,4 +99,4 @@ class MiniApplicationAdmin( MyApplicationAdmin ):
         from camelot.view import art
         return art.read('stylesheet/black.qss')
     
-# end mini admin
\ No newline at end of file
+# end mini admin
