@@ -5,10 +5,10 @@ import os
 def load_movie_fixtures():
 
     from camelot.model.fixture import Fixture
-    from camelot.model.authentication import Person
+    from camelot.model.party import Person
     from camelot_example.model import Movie, VisitorReport
     from camelot.core.files.storage import Storage, StoredImage
-    from camelot.core.resources import resource_string
+    from camelot.core.resources import resource_filename
 
     storage = Storage(upload_to='covers',
                       stored_file_implementation = StoredImage)
@@ -29,7 +29,7 @@ def load_movie_fixtures():
             [u'Horror',u'Mystery',u'Thriller'],
             u'thriller',
             4,
-            u'shining.jpg',
+            u'shining.png',
             u'A family heads to an isolated hotel for the winter where an evil'
             ' and spiritual presence influences the father into violence,'
             ' while his psychic son sees horrific forebodings from the past'
@@ -50,7 +50,7 @@ def load_movie_fixtures():
             [u'Action',u'Adventure'],
             u'action',
             4,
-            u'bourne.jpg',
+            u'bourne.png',
             u'A man is picked up by a fishing boat, bullet-riddled and without'
             ' memory, then races to elude assassins and recover from amnesia.'
         ],
@@ -70,7 +70,7 @@ def load_movie_fixtures():
             [u'Action',u'Adventure'],
             u'action',
             5,
-            u'casino.jpg',
+            u'casino.png',
             u"In his first mission, James Bond must stop Le Chiffre, a banker"
             " to the world's terrorist organizations, from winning a"
             " high-stakes poker tournament at Casino Royale in Montenegro."
@@ -90,7 +90,7 @@ def load_movie_fixtures():
             [u'Animation',u'Adventure'],
             u'animation',
             4,
-            u'toystory.jpg',
+            u'toystory.png',
             u"a cowboy toy is profoundly threatened and jealous when a fancy"
             " spaceman toy supplants him as top toy in a boy's room."
         ],
@@ -109,7 +109,7 @@ def load_movie_fixtures():
             [u'Family',u'Adventure'],
             u'family',
             3,
-            u'potter.jpg',
+            u'potter.png',
             u'Rescued from the outrageous neglect of his aunt and uncle, a'
             ' young boy with a great destiny proves his worth while attending'
             ' Hogwarts School of Witchcraft and Wizardry.'
@@ -129,7 +129,7 @@ def load_movie_fixtures():
             [u'Action',u'Adventure',u'Sci-fi'],
             u'sci-fi',
             3,
-            u'ironman.jpg',
+            u'ironman.png',
             u'billionaire Tony Stark must contend with deadly issues involving'
             ' the government, his own friends, as well as new enemies due to'
             ' his superhero alter ego Iron Man.'
@@ -150,7 +150,7 @@ def load_movie_fixtures():
             [u'Animation',u'Adventure'],
             u'animation',
             5,
-            u'lionking.jpg',
+            u'lionking.png',
             u'Tricked into thinking he killed his father, a guilt ridden lion'
             ' cub flees into exile and abandons his identity as the future'
             ' King.'
@@ -170,7 +170,7 @@ def load_movie_fixtures():
             [u'Action',u'Adventure',u'Sci-fi'],
             u'sci-fi',
             5,
-            u'avatar.jpg',
+            u'avatar.png',
             u'A paraplegic marine dispatched to the moon Pandora on a unique'
             ' mission becomes torn between following his orders and'
             ' protecting the world he feels is his home.'
@@ -190,7 +190,7 @@ def load_movie_fixtures():
             [u'Action',u'Adventure'],
             u'action',
             5,
-            u'pirates.jpg',
+            u'pirates.png',
             u"Blacksmith Will Turner teams up with eccentric pirate \"Captain\""
             " Jack Sparrow to save his love, the governor's daughter, from"
             " Jack's former pirate allies, who are now undead."
@@ -210,7 +210,7 @@ def load_movie_fixtures():
             [u'Action',u'Drama'],
             u'action',
             5,
-            u'darkknight.jpg',
+            u'darkknight.png',
             u'Batman, Gordon and Harvey Dent are forced to deal with the chaos'
             ' unleashed by an anarchist mastermind known only as the Joker, as'
             ' it drives each of them to their limits.'
@@ -236,21 +236,19 @@ def load_movie_fixtures():
     }
 
     for title, short_description, releasedate, (director_first_name, director_last_name), cast, tags, genre, rating, cover, description in movies:
-        director = Fixture.insertOrUpdateFixture(
+        director = Fixture.insert_or_update_fixture(
             Person,
             fixture_key = u'%s_%s'%(director_first_name, director_last_name),
             values = {'first_name':director_first_name,
                       'last_name':director_last_name}
         )
-        movie = Fixture.findFixture( Movie, title )
+        movie = Fixture.find_fixture( Movie, title )
         if not movie:
-            image = resource_string( 'camelot_example', os.path.join( 'media', 'covers', cover ) )
-            stream = StringIO.StringIO()
-            stream.write( image )
-            stream.seek( 0 )
-            prefix, suffix = os.path.splitext( cover )
-            stored_image = storage.checkin_stream( prefix, suffix, stream )
-            movie = Fixture.insertOrUpdateFixture(
+            # use resource_filename, since resource_string seems to mess either with encoding
+            # or with line endings, when on windows
+            image = resource_filename( 'camelot_example', os.path.join( 'media', 'covers', cover ) )
+            stored_image = storage.checkin( image )
+            movie = Fixture.insert_or_update_fixture(
                 Movie,
                 fixture_key = title,
                 values = {
@@ -267,7 +265,7 @@ def load_movie_fixtures():
         rep = visits.get(title, None)
         if rep:
             for city, visitors, date in rep:
-                Fixture.insertOrUpdateFixture(
+                Fixture.insert_or_update_fixture(
                     VisitorReport,
                     fixture_key = '%s_%s' % (title, city),
                     values = {
