@@ -1,10 +1,12 @@
+import datetime
 import os
 
-from PyQt4 import QtGui
+from PyQt4 import QtGui, QtCore
+from PyQt4.QtCore import Qt
 
 from camelot.admin.action import Action
 from camelot.admin.action import list_action, application_action
-from camelot.core.utils import ugettext_lazy as _
+from camelot.core.utils import pyqt, ugettext_lazy as _
 from camelot.test import ModelThreadTestCase
 from camelot.test.action import MockModelContext
 from camelot.view import action_steps
@@ -63,9 +65,9 @@ class ActionStepsCase( ModelThreadTestCase ):
     images_path = static_images_path
     
     def setUp(self):
+        ModelThreadTestCase.setUp(self)
         from camelot_example.model import Movie
         from camelot.admin.application_admin import ApplicationAdmin
-        ModelThreadTestCase.setUp(self)
         self.app_admin = ApplicationAdmin()
         self.context = MockModelContext()
         self.context.obj = Movie.query.first()
@@ -82,7 +84,7 @@ class ActionStepsCase( ModelThreadTestCase ):
         select_file = generator.next()
         self.assertFalse( select_file.single )
         # pretend the user selected a file
-        generator.send( [os.path.join( os.path.dirname(__file__), '..', 'camelot_example', 'media', 'covers', 'circus.jpg') ] )
+        generator.send( [os.path.join( os.path.dirname(__file__), '..', 'camelot_example', 'media', 'covers', 'circus.png') ] )
         # continue the action till the end
         list( generator )
         # a movie should be inserted
@@ -155,14 +157,29 @@ class ListActionsCase( ModelThreadTestCase ):
     images_path = static_images_path
 
     def setUp( self ):
+        ModelThreadTestCase.setUp(self)
         from camelot_example.model import Movie
         from camelot.admin.application_admin import ApplicationAdmin
-        ModelThreadTestCase.setUp(self)
         self.app_admin = ApplicationAdmin()
         self.context = MockModelContext()
         self.context.obj = Movie.query.first()
         self.context.admin = self.app_admin.get_related_admin( Movie )
         
+    def test_sqlalchemy_command( self ):
+        model_context = self.context
+        from camelot.model.batch_job import BatchJobType
+        # create a batch job to test with
+        bt = BatchJobType( name = 'audit' )
+        model_context.session.add( bt )
+        bt.flush()
+        # begin issue a query through the model_context
+        model_context.session.query( BatchJobType ).update( values = {'name':'accounting audit'},
+                                                            synchronize_session = 'evaluate' )
+        # end issue a query through the model_context
+        #
+        # the batch job should have changed
+        self.assertEqual( bt.name, 'accounting audit' )
+        
     def test_change_row_actions( self ):
         from camelot.test.action import MockListActionGuiContext
         
@@ -208,7 +225,7 @@ class ListActionsCase( ModelThreadTestCase ):
         self.test_import_from_file( 'import_example.xls' )
 
     def test_import_from_file( self, filename = 'import_example.csv' ):
-        from camelot.model.authentication import Person
+        from camelot.model.party import Person
         example_folder = os.path.join( os.path.dirname(__file__), '..', 'camelot_example' )
         self.context = MockModelContext()
         self.context.obj = Person.query.first() # need an object, to have a
@@ -238,7 +255,25 @@ class ListActionsCase( ModelThreadTestCase ):
                 dialog.show()
                 self.grab_widget( dialog ) 
                 generator.send( ('rating', lambda:3) )
-
+                
+    def test_drag_and_drop( self ):
+        from camelot.view.proxy.queryproxy import QueryTableProxy
+        
+        class DropAction( Action ):
+            pass
+                
+        
+        mime_data = QtCore.QMimeData()
+        admin = self.context.admin
+        admin.drop_action = DropAction()
+        
+        proxy = QueryTableProxy( admin, admin.get_query, admin.get_columns )
+        proxy.dropMimeData( mime_data, 
+                            Qt.MoveAction, 
+                            -1, 
+                            -1, 
+                            QtCore.QModelIndex() )
+        
 class ApplicationActionsCase( ModelThreadTestCase ):
     """Test application actions.
     """
@@ -254,8 +289,34 @@ class ApplicationActionsCase( ModelThreadTestCase ):
         self.storage = Storage()
 
     def test_refresh( self ):
+        from camelot.core.orm import Session
+        from camelot.model.party import Person
         refresh_action = application_action.Refresh()
+        session = Session()
+        #
+        # create objects in various states
+        #
+        p1 = Person(first_name = 'p1', last_name = 'persistent' )
+        p2 = Person(first_name = 'p2', last_name = 'dirty' )
+        p3 = Person(first_name = 'p3', last_name = 'deleted' )
+        p4 = Person(first_name = 'p4', last_name = 'to be deleted' )
+        p5 = Person(first_name = 'p5', last_name = 'detached' )
+        p6 = Person(first_name = 'p6', last_name = 'deleted outside session' )
+        session.flush()
+        p3.delete()
+        session.flush()
+        p4.delete()
+        p2.last_name = 'clean'
+        #
+        # delete p6 without the session being aware
+        #
+        person_table = Person.table
+        session.execute( person_table.delete().where( person_table.c.party_id == p6.id ) )
+        #
+        # refresh the session through the action
+        #
         list( refresh_action.model_run( self.context ) )
+        self.assertEqual( p2.last_name, 'dirty' )
         
     def test_backup_and_restore( self ):
         backup_action = application_action.Backup()
