@@ -7,7 +7,8 @@ import os
 import time
 
 from camelot.core.utils import ugettext_lazy as _
-from camelot.test import ModelThreadTestCase, EntityViewsTest, SchemaTest
+from camelot.core.files.storage import StoredFile, StoredImage, Storage
+from camelot.test import ModelThreadTestCase, EntityViewsTest
 from camelot.view.art import ColorScheme
 
 from PyQt4 import QtGui, QtCore
@@ -17,6 +18,7 @@ from PyQt4.QtCore import *
 logger = logging.getLogger('view.unittests')
 
 static_images_path = os.path.join(os.path.dirname(__file__), '..', 'doc', 'sphinx', 'source', '_static')
+storage = Storage()
 
 def create_getter(getable):
 
@@ -25,148 +27,16 @@ def create_getter(getable):
 
     return getter
 
-#class ProxyEntityTest(ModelThreadTestCase):
-#  """Test the functionality of the proxies to perform CRUD operations on stand
-#  alone data"""
-#  def setUp(self):
-#    ModelThreadTestCase.setUp(self)
-#    from camelot.model.authentication import Person, PartyAddressRoleType
-#    from camelot.view.proxy.queryproxy import QueryTableProxy
-#    from camelot.view.application_admin import ApplicationAdmin
-#    self.app_admin = ApplicationAdmin()
-#    self.block = self.mt.post_and_block
-#    person_admin = self.app_admin.getEntityAdmin(Person)
-#    party_address_role_type_admin = self.app_admin.getEntityAdmin(PartyAddressRoleType)
-#    self.person_proxy = QueryTableProxy(person_admin, lambda:Person.query, person_admin.get_fields)
-#    self.party_address_role_type_proxy = QueryTableProxy(party_address_role_type_admin, lambda:PartyAddressRoleType.query, party_address_role_type_admin.get_fields)
-#    # get the columns of the proxy
-#    self.person_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.person_proxy.getColumns())))
-#    self.rows_before_insert = 0
-#    self.rows_after_insert = 0
-#    self.rows_after_delete = 0
-#    self.new_person = None
-#    # make sure nothing is running in the model thread any more
-#    self.block(lambda:None)
-#  def listPersons(self):
-#    from camelot.model.authentication import Person
-#
-#    def create_list():
-#      print '<persons>'
-#      for p in Person.query.all():
-#        print p.id, unicode(p)
-#      print '</persons>'
-#
-#    self.block(create_list)
-#  def numberOfPersons(self):
-#    return self.block(lambda:self.person_proxy.getRowCount())
-#  def insertNewPerson(self, valid):
-#    from camelot.model.authentication import Person
-#    self.rows_before_insert = self.numberOfPersons()
-#    self.new_person = self.block(lambda:Person(first_name={True:u'test', False:None}[valid], last_name='test'))
-#    self.person_proxy.insertRow(0, create_getter(self.new_person))
-#    self.rows_after_insert = self.numberOfPersons()
-#    self.assertTrue( self.rows_after_insert > self.rows_before_insert)
-#  def updateNewPerson(self):
-#    rows_before_update = self.numberOfPersons()
-#    self.assertNotEqual( self.block(lambda:self.new_person.last_name), u'Testers')
-#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
-#    self.person_proxy.setData(index, lambda:u'Testers')
-#    self.assertEqual( self.block(lambda:self.new_person.last_name), u'Testers')
-#    self.assertEqual(rows_before_update, self.numberOfPersons())
-#  def updateNewPersonToValid(self):
-#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
-#    self.person_proxy.setData(index, lambda:u'test')
-#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
-#    self.person_proxy.setData(index, lambda:u'test')
-#  def updateNewPersonToInvalid(self):
-#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
-#    self.person_proxy.setData(index, lambda:None)
-#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
-#    self.person_proxy.setData(index, lambda:None)
-#  def deleteNewPerson(self):
-#    self.person_proxy.removeRow(self.rows_before_insert)
-#    self.rows_after_delete = self.numberOfPersons()
-#    self.assertTrue( self.rows_after_delete < self.rows_after_insert )
-#  def testCreateDataWithoutRequiredFields(self):
-#    """Create a record that has no required fields, so the creation
-#    should be instantaneous as opposed to delayed until all required fields are filled"""
-#    from camelot.model.authentication import PartyAddressRoleType
-#    rows_before_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
-#    new_party_address_role_type_proxy = self.block(lambda:PartyAddressRoleType())
-#    self.party_address_role_type_proxy.insertRow(0, create_getter(new_party_address_role_type_proxy))
-#    self.assertNotEqual( self.block(lambda:new_party_address_role_type_proxy.id), None )
-#    rows_after_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
-#    self.assertTrue( rows_after_insert > rows_before_insert)
-#  def testCreateUpdateDeleteValidData(self):
-#    self.insertNewPerson(valid=True)
-#    self.assertTrue( self.block(lambda:self.new_person.id) != None )
-#    self.updateNewPerson()
-#    self.deleteNewPerson()
-#  def testCreateUpdateDeleteInvalidData(self):
-#    self.insertNewPerson(valid=False)
-#    self.assertTrue( self.block(lambda:self.new_person.id) == None )
-#    self.updateNewPerson()
-#    self.deleteNewPerson()
-#  def testCreateInvalidDataUpdateToValidAndDelete(self):
-#    self.insertNewPerson(valid=False)
-#    self.assertTrue( self.block(lambda:self.new_person.id) == None )
-#    self.updateNewPersonToValid()
-#    self.assertTrue( self.block(lambda:self.new_person.id) != None )
-#    self.deleteNewPerson()
-#  def testCreateValidDataUpdateToInvalidAndDelete(self):
-#    self.insertNewPerson(valid=False)
-#    self.updateNewPersonToInvalid()
-#    self.deleteNewPerson()
-
-
-#class ProxyOneToManyTest(ProxyEntityTest):
-#  """Test the functionality of the proxies to perform CRUD operations on related
-#  data"""
-#  def setUp(self):
-#    super(ProxyOneToManyTest, self).setUp()
-#    from elixir import session
-#    from camelot.view.proxy.queryproxy import QueryTableProxy
-#    from camelot.model.authentication import Country, City, Address, PartyAddress
-#    party_address_admin = self.app_admin.getEntityAdmin(PartyAddress)
-#    self.party_address_proxy = QueryTableProxy(party_address_admin, PartyAddress.query, party_address_admin.get_fields)
-#    self.party_address_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.party_address_proxy.getColumns())))
-#    self.country = self.block(lambda:Country(code=u'BE', name=u'Belgium'))
-#    self.city = self.block(lambda:City(code=u'2000', name=u'Antwerp', country=self.country))
-#    self.address = self.block(lambda:Address(street1=u'Teststreet', city=self.city))
-#    self.block(lambda:session.flush([self.country, self.city, self.address]))
-#  def insertRelatedPartyAddress(self, valid):
-#    from camelot.model.authentication import PartyAddress
-#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['addresses'])
-#    self.block(lambda:None)
-#    self.related_party_address_proxy = self.person_proxy.data(index, Qt.EditRole).toPyObject()
-#    self.block(lambda:None)
-#    self.related_party_address_proxy = self.person_proxy.data(index, Qt.EditRole).toPyObject()
-#    self.assertNotEqual(self.related_party_address_proxy, None)
-#    self.party_address = self.block({True:lambda:PartyAddress(address=self.address), False:lambda:PartyAddress()}[valid])
-#    self.related_party_address_proxy.insertRow(0, create_getter(self.party_address))
-#    self.block(lambda:None)
-#  def testCreateBothValidDataUpdateDelete(self):
-#    self.insertNewPerson(valid=True)
-#    self.insertRelatedPartyAddress(valid=True)
-#    self.assertNotEqual(self.party_address.id, None)
-#    self.deleteNewPerson()
-#  def testCreateValidPersonInvalidPartyAddressUpdateDelete(self):
-#    self.insertNewPerson(valid=True)
-#    self.insertRelatedPartyAddress(valid=False)
-#    self.assertEqual(self.party_address.id, None)
-#    self.deleteNewPerson()
-#  def testCreateInvalidPersonValidPartyAddressUpdateDelete(self):
-#    self.insertNewPerson(valid=False)
-#    self.insertRelatedPartyAddress(valid=False)
-#    self.assertEqual(self.party_address.id, None)
-#    self.deleteNewPerson()
-#  def tearDown(self):
-#    from elixir import session
-#    self.block(lambda:self.address.delete())
-#    self.block(lambda:self.city.delete())
-#    self.block(lambda:self.country.delete())
-#    self.block(lambda:session.flush([self.country, self.city, self.address]))
-
+class SignalCounter( QtCore.QObject ):
+    
+    def __init__( self ):
+        super( SignalCounter, self ).__init__()
+        self.counter = 0
+        
+    @QtCore.pyqtSlot()
+    def signal_caught( self ):
+        self.counter += 1
+        
 class EditorsTest(ModelThreadTestCase):
     """
   Test the basic functionality of the editors :
@@ -181,6 +51,33 @@ class EditorsTest(ModelThreadTestCase):
     from camelot.view.controls import editors
     from camelot.view.proxy import ValueLoading
 
+    def assert_valid_editor( self, editor, value ):
+        """Test the basic functions of an editor that are needed to integrate
+        well with Camelot and Qt
+        """
+        #
+        # The editor should remember its when its value is ValueLoading
+        #
+        editor.set_value( self.ValueLoading )
+        self.assertEqual( editor.get_value(), self.ValueLoading )
+        #
+        # When a value is set, no editingFinished should be called
+        #
+        signal_counter = SignalCounter()
+        editor.editingFinished.connect( signal_counter.signal_caught )
+        editor.set_value( value )
+        self.assertEqual( signal_counter.counter, 0 )
+        #
+        # when the up or down arrow is pressed, the event should be ignored
+        # by the editor, to allow the table view to move to the row above or
+        # below
+        #
+        #up_event = QtGui.QKeyEvent( QtCore.QEvent.KeyPress, 
+        #                            Qt.Key_Up,
+        #                            Qt.NoModifier )
+        #editor.keyPressEvent( up_event )
+        #self.assertFalse( up_event.isAccepted() )
+        
     def test_ChartEditor(self):
         import math
         from camelot.container import chartcontainer
@@ -193,6 +90,7 @@ class EditorsTest(ModelThreadTestCase):
         self.grab_widget( editor, 'editable' )
         editor.set_field_attributes(editable=False)
         self.grab_widget( editor, 'disabled' )
+        self.assert_valid_editor( editor, plot )
         
     def test_DateEditor(self):
         import datetime
@@ -203,8 +101,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( datetime.date(1980, 12, 31) )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), datetime.date(1980, 12, 31) )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, datetime.date(1980, 12, 31) )
 
     def test_TextLineEditor(self):
         editor = self.editors.TextLineEditor(parent=None, length=10)
@@ -219,13 +116,12 @@ class EditorsTest(ModelThreadTestCase):
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( u'za coś tam' )
         self.assertEqual( editor.get_value(), u'za coś tam' )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( None )
         self.assertEqual( editor.get_value(), None )
         # pretend the user has entered some text
         editor.setText( u'foo' )
         self.assertTrue( editor.get_value() != None )
+        self.assert_valid_editor( editor, u'za coś tam' )
         
     def grab_default_states( self, editor ):
         editor.set_field_attributes( editable = True, background_color=ColorScheme.green )
@@ -252,8 +148,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( '/home/lancelot/quests.txt' )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), '/home/lancelot/quests.txt' )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, '/home/lancelot/quests.txt' )
         
     def test_StarEditor(self):
         editor = self.editors.StarEditor(parent=None, maximum=5)
@@ -261,8 +156,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 4 )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 4 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, 4 )
 
     def test_SmileyEditor(self):
         editor = self.editors.SmileyEditor(parent=None)
@@ -270,8 +164,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 'face-kiss' )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 'face-kiss' )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, 'face-kiss' )
         
     def test_BoolEditor(self):
         editor = self.editors.BoolEditor(parent=None, editable=False, nullable=True)
@@ -289,10 +182,13 @@ class EditorsTest(ModelThreadTestCase):
         self.assertEqual( editor.get_value(), True )
         editor.set_value( False )
         self.assertEqual( editor.get_value(), False )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        # changing the editable state should preserve the value
         editor.set_value( True )
-        editor.set_enabled( True )
+        editor.set_field_attributes( editable = False )
+        self.assertEqual( editor.get_value(), True )
+        editor.set_field_attributes( editable = True )
+        self.assertEqual( editor.get_value(), True )        
+        self.assert_valid_editor( editor, True )
 
     def test_CodeEditor(self):
         editor = self.editors.CodeEditor(parent=None, parts=['AAA', '999'])
@@ -300,10 +196,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( ['XYZ', '123'] )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), ['XYZ', '123'] )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, ['XYZ', '123'] )
 
     def test_ColorEditor(self):
         editor = self.editors.ColorEditor(parent=None, editable=True)
@@ -311,8 +204,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( (255, 200, 255, 255) )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), (255, 200, 255, 255) )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, (255, 200, 255, 255) )
 
     def test_ColoredFloatEditor(self):
         editor = self.editors.ColoredFloatEditor(parent=None, editable=True)
@@ -322,8 +214,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 3.14 )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 3.14 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, 3.14 )
 
     def test_ChoicesEditor(self):
         editor = self.editors.ChoicesEditor(parent=None, editable=True)
@@ -337,16 +228,23 @@ class EditorsTest(ModelThreadTestCase):
         # now change the choices
         choices2 = [(4,u'D'), (5,u'E'), (6,u'F')]
         editor.set_choices( choices2 )
-        self.assertEqual(editor.get_choices(), choices2 + [(2,u'B')])
-        editor.set_value( self.ValueLoading )
+        self.assertEqual( editor.get_choices(), choices2 + [(2,u'B')] )
+        # set a value that is not in the list, the value should become
+        # ValueLoading, to prevent damage to the actual data
+        editor.set_value( 33 )
         self.assertEqual( editor.get_value(), self.ValueLoading )
+        # try strings as keys
+        editor = self.editors.ChoicesEditor(parent=None, editable=True)
+        editor.set_choices( [('a',u'A'), ('b',u'B'), ('c',u'C')] )
+        editor.set_value( 'c' )
+        self.assertEqual( editor.get_value(), 'c' )
+        self.assert_valid_editor( editor, 'c' )
 
     def test_FileEditor(self):
         editor = self.editors.FileEditor(parent=None, editable=True)
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
         self.grab_default_states( editor )
+        self.assert_valid_editor( editor, StoredFile( storage, 'test.txt') )
 
     def test_DateTimeEditor(self):
         import datetime
@@ -355,8 +253,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( datetime.datetime(2009, 7, 19, 21, 5, 10, 0) )
         self.assertEqual( editor.get_value(), datetime.datetime(2009, 7, 19, 21, 5, 0 ) )
         self.grab_default_states( editor )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, datetime.datetime(2009, 7, 19, 21, 5, 0 ) )
 
     def test_FloatEditor(self):
         editor = self.editors.FloatEditor(parent=None, 
@@ -367,8 +264,6 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 3.14 )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 3.14 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
         editor = self.editors.FloatEditor(parent=None,  
                                           suffix='suffix')
         self.assertEqual( editor.get_value(), self.ValueLoading )
@@ -376,8 +271,6 @@ class EditorsTest(ModelThreadTestCase):
         self.assertEqual( editor.get_value(), 0.0 )
         editor.set_value( 3.14 )
         self.assertEqual( editor.get_value(), 3.14 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( 5.45 )
         editor.set_value( None )
         self.assertEqual( editor.get_value(), None )
@@ -391,13 +284,13 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 3.14 )
         self.grab_widget( editor, 'no_calculator' )
         self.assertTrue( editor.calculatorButton.isHidden() )
+        self.assert_valid_editor( editor, 3.14 )
         
     def test_ImageEditor(self):
         editor = self.editors.ImageEditor(parent=None, editable=True)
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
         self.grab_default_states( editor )
+        self.assert_valid_editor( editor, StoredImage( storage, 'test.png') )
 
     def test_IntegerEditor(self):
         editor = self.editors.IntegerEditor(parent=None, editable=True)
@@ -407,8 +300,6 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 3 )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 3 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( None )
         # pretend the user changed the value
         editor.spinBox.setValue( 0 )
@@ -419,15 +310,17 @@ class EditorsTest(ModelThreadTestCase):
         editor = self.editors.IntegerEditor(parent=None, 
                                             calculator=False)
         editor.set_field_attributes( editable=True )
-        editor.set_value( 3.14 )
+        editor.set_value( 3 )
         self.grab_widget( editor, 'no_calculator' )
         self.assertTrue( editor.calculatorButton.isHidden() )
+        self.assert_valid_editor( editor, 3 )
         
     def test_NoteEditor(self):
         editor = self.editors.NoteEditor(parent=None)
         editor.set_value('A person with this name already exists')
         self.grab_widget( editor )
         self.grab_default_states( editor )
+        self.assert_valid_editor( editor, 'A person with this name already exists' )
 
     def test_LabelEditor(self):
         editor = self.editors.LabelEditor(parent=None)
@@ -440,12 +333,12 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 'en_US' )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 'en_US' )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, 'en_US' )
 
     def test_Many2OneEditor(self):
         editor = self.editors.Many2OneEditor(parent=None)
         self.grab_default_states( editor )
+        self.assert_valid_editor( editor, lambda:object )
         
     def test_RichTextEditor(self):
         editor = self.editors.RichTextEditor(parent=None)
@@ -453,8 +346,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( u'<h1>Rich Text Editor</h1>' )
         self.grab_default_states( editor )
         self.assertTrue( u'Rich Text Editor' in editor.get_value() )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, u'<h1>Rich Text Editor</h1>' )
 
     def test_TimeEditor(self):
         import datetime
@@ -463,8 +355,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( datetime.time(21, 5, 0) )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), datetime.time(21, 5, 0) )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, datetime.time(21, 5, 0) )
 
     def test_TextEditEditor(self):
         editor = self.editors.TextEditEditor(parent=None, editable=True)
@@ -472,8 +363,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 'Plain text' )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 'Plain text' )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, 'Plain text' )
         
     def test_VirtualAddressEditor(self):
         editor = self.editors.VirtualAddressEditor(parent=None)
@@ -481,8 +371,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( ('im','test') )
         self.grab_default_states( editor )
         self.assertEqual( editor.get_value(),  ('im','test') )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
+        self.assert_valid_editor( editor, ('im','test') )
 
     def test_MonthsEditor(self):
         editor = self.editors.MonthsEditor(parent=None)
@@ -490,8 +379,7 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value(12)
         self.grab_default_states( editor )
         self.assertEqual(editor.get_value(),  12)
-        editor.set_value(self.ValueLoading)
-        self.assertEqual(editor.get_value(), self.ValueLoading)
+        self.assert_valid_editor( editor, 12 )
 
 from camelot.view import forms
 
@@ -506,7 +394,7 @@ class FormTest(ModelThreadTestCase):
         self.entities = [e for e in entities]
         from camelot.admin.application_admin import ApplicationAdmin
         from camelot.view.proxy.queryproxy import QueryTableProxy
-        from camelot.model.authentication import Person
+        from camelot.model.party import Person
         from camelot_example.model import Movie
         self.app_admin = ApplicationAdmin()
         self.movie_admin = self.app_admin.get_related_admin( Movie )
@@ -657,6 +545,11 @@ class DelegateTest(ModelThreadTestCase):
         editor = delegate.createEditor(None, self.option, None)
         self.assertTrue(isinstance(editor, self.editors.TextLineEditor))
         self.grab_delegate(delegate, 'Plain Text', 'disabled')
+        small_text_delegate = self.delegates.PlainTextDelegate( length = 3 )
+        wide_text_delegate = self.delegates.PlainTextDelegate( length = 30 )
+        small_size = small_text_delegate.sizeHint( None, 0 ).width()
+        wide_size = wide_text_delegate.sizeHint( None, 0 ).width()
+        self.assertTrue( small_size < wide_size )
 
     def testTextEditDelegate(self):
         from PyQt4.QtGui import QTextEdit
@@ -936,7 +829,7 @@ class FilterTest(ModelThreadTestCase):
 
     def test_filter_list_in_table_view(self):
         from camelot.view.controls.tableview import TableView
-        from camelot.model.authentication import Person
+        from camelot.model.party import Person
         from camelot.admin.application_admin import ApplicationAdmin
         from camelot.admin.action.base import GuiContext
         gui_context = GuiContext()
@@ -968,13 +861,75 @@ class ControlsTest(ModelThreadTestCase):
         
     def test_table_view(self):
         from camelot.view.controls.tableview import TableView
-        from camelot.model.authentication import Person
+        from camelot.model.party import Person
         from camelot.admin.action.base import GuiContext
         gui_context = GuiContext()
         widget = TableView( gui_context, 
                             self.app_admin.get_entity_admin(Person) )
         self.grab_widget(widget)
+        
+    def test_small_column( self ):
+        #create a table view for an Admin interface with small columns
+        from camelot.view.controls.tableview import TableView
+        from camelot.model.party import Person
+        
+        class SmallColumnsAdmin( Person.Admin ):
+            list_display = ['first_name', 'suffix']
+            
+        admin = SmallColumnsAdmin( self.app_admin, Person )
+        widget = TableView( self.gui_context, 
+                            admin )
+        self.grab_widget( widget )
+        model = widget.table.model()
+        header = widget.table.horizontalHeader()
 
+        first_name_width = model.headerData( 0, Qt.Horizontal, Qt.SizeHintRole ).toSize().width()
+        suffix_width = model.headerData( 1, Qt.Horizontal, Qt.SizeHintRole ).toSize().width()
+        
+        self.assertTrue( first_name_width > suffix_width )
+        
+    def test_column_width( self ):
+        #create a table view for an Admin interface with small columns
+        from camelot.view.controls.tableview import TableView
+        from camelot.model.party import Person
+        
+        class ColumnWidthAdmin( Person.Admin ):
+            list_display = ['first_name', 'suffix']
+            # begin column width
+            field_attributes = { 'first_name':{'column_width':8},
+                                 'suffix':{'column_width':8},}
+            # end column width
+            
+        admin = ColumnWidthAdmin( self.app_admin, Person )
+        widget = TableView( self.gui_context, 
+                            admin )
+        self.grab_widget( widget )
+        model = widget.table.model()
+        header = widget.table.horizontalHeader()
+
+        first_name_width = model.headerData( 0, Qt.Horizontal, Qt.SizeHintRole ).toSize().width()
+        suffix_width = model.headerData( 1, Qt.Horizontal, Qt.SizeHintRole ).toSize().width()
+        
+        self.assertEqual( first_name_width, suffix_width )
+        
+    def test_column_group( self ):
+        from camelot.admin.table import ColumnGroup
+        from camelot.view.controls.tableview import TableView
+        from camelot.model.party import Person
+
+        class ColumnWidthAdmin( Person.Admin ):
+            #begin column group
+            list_display = [ ColumnGroup( _('Name'), ['first_name', 'last_name', 'suffix'] ),
+                             ColumnGroup( _('Official'), ['birthdate', 'social_security_number', 'passport_number'] ),
+                             ]
+            #end column group
+            
+        admin = ColumnWidthAdmin( self.app_admin, Person )
+        widget = TableView( self.gui_context, 
+                            admin )
+        widget.setMinimumWidth( 800 )
+        self.grab_widget( widget )
+        
     def test_navigation_pane(self):
         from camelot.view.controls import navpane2
         self.wait_for_animation()
@@ -1003,11 +958,11 @@ class ControlsTest(ModelThreadTestCase):
         self.wait_for_animation()
         self.grab_widget( widget )     
 
-    def test_status_bar(self):
-        from camelot.view.controls.statusbar import StatusBar
-        status_bar = StatusBar(None)
-        status_bar.busy_widget.set_busy(True)
-        self.grab_widget(status_bar)
+    def test_busy_widget(self):
+        from camelot.view.controls.busy_widget import BusyWidget
+        busy_widget = BusyWidget()
+        busy_widget.set_busy( True )
+        self.grab_widget( busy_widget )
 
     def test_search_control(self):
         from camelot.view.controls.search import SimpleSearchControl
@@ -1015,13 +970,40 @@ class ControlsTest(ModelThreadTestCase):
         self.grab_widget(search)
 
     def test_header_widget(self):
-        from camelot.model.authentication import City
+        from camelot.model.party import City
         from camelot.view.controls.tableview import HeaderWidget
         person_admin = self.app_admin.get_entity_admin(City)
         header = HeaderWidget(parent=None, admin=person_admin)
         header.expand_search_options()
         self.grab_widget(header)
         
+    def test_column_groups_widget(self):
+        from camelot.view.controls.tableview import ColumnGroupsWidget
+        from camelot_example.view import VisitorsPerDirector
+        table = VisitorsPerDirector.Admin.list_display
+        widget = QtGui.QWidget()
+        layout = QtGui.QVBoxLayout()
+        table_widget = QtGui.QTableWidget( 3, 6 )
+        table_widget.setHorizontalHeaderLabels( table.get_fields() )
+        column_groups = ColumnGroupsWidget( table,
+                                            table_widget )
+        layout.addWidget( table_widget )
+        layout.addWidget( column_groups )
+        widget.setLayout( layout )
+        #
+        # set the tab to 1 and then back to 0, to force a change
+        # signal
+        #
+        column_groups.setCurrentIndex( 1 )
+        column_groups.setCurrentIndex( 0 )
+        self.assertFalse( table_widget.isColumnHidden( 0 ) )
+        self.assertTrue( table_widget.isColumnHidden( 3 ) )
+        self.grab_widget( widget, 'first_tab' )
+        column_groups.setCurrentIndex( 1 )
+        self.assertTrue( table_widget.isColumnHidden( 0 ) )
+        self.assertFalse( table_widget.isColumnHidden( 3 ) )
+        self.grab_widget( widget, 'second_tab' )
+        
     def test_desktop_workspace(self):
         from camelot.view.workspace import DesktopWorkspace
         from camelot.admin.action import Action
@@ -1069,11 +1051,16 @@ class SnippetsTest(ModelThreadTestCase):
 
     images_path = static_images_path
 
+    def setUp( self ):
+        super( SnippetsTest, self ).setUp()
+        from camelot.admin.application_admin import ApplicationAdmin
+        self.app_admin = ApplicationAdmin()
+        
     def test_simple_plot(self):
         from snippet.chart.simple_plot import Wave
         from camelot.view.proxy.collection_proxy import CollectionProxy
         wave = Wave()
-        admin = Wave.Admin(None, Wave)
+        admin = Wave.Admin( self.app_admin, Wave )
         proxy = CollectionProxy(admin, lambda:[wave], admin.get_fields )
         form = admin.create_form_view('Wave', proxy, 0, None)
         form.setMaximumSize( 400, 200 )
@@ -1084,7 +1071,7 @@ class SnippetsTest(ModelThreadTestCase):
         from camelot.view.proxy.collection_proxy import CollectionProxy
         wave = Wave()
         #wave.phase = '2.89'
-        admin = Wave.Admin(object(), Wave)
+        admin = Wave.Admin( self.app_admin, Wave )
         proxy = CollectionProxy(admin, lambda:[wave], admin.get_fields )
         form = admin.create_form_view('Wave', proxy, 0, None)
         form.setMaximumSize( 400, 200 )
@@ -1094,7 +1081,7 @@ class SnippetsTest(ModelThreadTestCase):
         from snippet.fields_with_actions import Coordinate
         from camelot.view.proxy.collection_proxy import CollectionProxy
         coordinate = Coordinate()
-        admin = Coordinate.Admin(None, Coordinate)
+        admin = Coordinate.Admin( self.app_admin, Coordinate )
         proxy = CollectionProxy(admin, lambda:[coordinate], admin.get_fields )
         form = admin.create_form_view('Coordinate', proxy, 0, None)
         self.grab_widget(form)
@@ -1103,18 +1090,16 @@ class SnippetsTest(ModelThreadTestCase):
         from snippet.fields_with_tooltips import Coordinate
         from camelot.view.proxy.collection_proxy import CollectionProxy
         coordinate = Coordinate()
-        admin = Coordinate.Admin(None, Coordinate)
+        admin = Coordinate.Admin( self.app_admin, Coordinate )
         proxy = CollectionProxy(admin, lambda:[coordinate], admin.get_fields )
         form = admin.create_form_view('Coordinate', proxy, 0, None)
         self.grab_widget(form)
 
     def test_entity_validator(self):
         from camelot.view.proxy.collection_proxy import CollectionProxy
-        from camelot.model.authentication import Person
-        from camelot.admin.application_admin import ApplicationAdmin
+        from camelot.model.party import Person
         from snippet.entity_validator import PersonValidator, Admin
-        app_admin = ApplicationAdmin()
-        person_admin = Admin(app_admin, Person)
+        person_admin = Admin( self.app_admin, Person)
         proxy = CollectionProxy(person_admin, lambda:[Person()], person_admin.get_columns)
         validator = PersonValidator(person_admin, proxy)
         self.mt.post(lambda:validator.isValid(0))
@@ -1124,11 +1109,9 @@ class SnippetsTest(ModelThreadTestCase):
 
     def test_background_color(self):
         from camelot.view.proxy.collection_proxy import CollectionProxy
-        from camelot.model.authentication import Person
-        from camelot.admin.application_admin import ApplicationAdmin
+        from camelot.model.party import Person
         from snippet.background_color import Admin
-        app_admin = ApplicationAdmin()
-        person_admin = Admin(app_admin, Person)
+        person_admin = Admin( self.app_admin, Person )
         proxy = CollectionProxy(person_admin, lambda:[Person(first_name='John', last_name='Cleese'),
                                                       Person(first_name='eric', last_name='Idle')],
                                                        person_admin.get_columns)
@@ -1137,8 +1120,3 @@ class SnippetsTest(ModelThreadTestCase):
         editor.set_value(proxy)
         self.process()
         self.grab_widget(editor)
-
-class CamelotSchemaTest(SchemaTest):
-
-    images_path = static_images_path
-
